@@ -1,18 +1,18 @@
-/* 
+/*
  * libdivecomputer
- * 
+ *
  * Copyright (C) 2008 Jef Driesen
- * 
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
@@ -146,6 +146,43 @@ array_convert_hex2bin (const unsigned char input[], unsigned int isize, unsigned
 	return 0;
 }
 
+unsigned int
+array_convert_str2num (const unsigned char data[], unsigned int size)
+{
+	unsigned int value = 0;
+	for (unsigned int i = 0; i < size; ++i) {
+		if (data[i] < '0' || data[i] > '9')
+			break;
+		value *= 10;
+		value += data[i] - '0';
+	}
+
+	return value;
+}
+
+unsigned int
+array_uint_be (const unsigned char data[], unsigned int n)
+{
+	unsigned int shift = n * 8;
+	unsigned int value = 0;
+	for (unsigned int i = 0; i < n; ++i) {
+		shift -= 8;
+		value |= data[i] << shift;
+	}
+	return value;
+}
+
+unsigned int
+array_uint_le (const unsigned char data[], unsigned int n)
+{
+	unsigned int shift = 0;
+	unsigned int value = 0;
+	for (unsigned int i = 0; i < n; ++i) {
+		value |= data[i] << shift;
+		shift += 8;
+	}
+	return value;
+}
 
 unsigned int
 array_uint32_be (const unsigned char data[])
@@ -161,6 +198,23 @@ array_uint32_le (const unsigned char data[])
 }
 
 
+unsigned int
+array_uint32_word_be (const unsigned char data[])
+{
+	return data[1] + (data[0] << 8) + (data[3] << 16) + (data[2] << 24);
+}
+
+
+void
+array_uint32_le_set (unsigned char data[], const unsigned int input)
+{
+	data[0] = input & 0xFF;
+	data[1] = (input >>  8) & 0xFF;
+	data[2] = (input >> 16) & 0xFF;
+	data[3] = (input >> 24) & 0xFF;
+}
+
+
 unsigned int
 array_uint24_be (const unsigned char data[])
 {
@@ -168,6 +222,15 @@ array_uint24_be (const unsigned char data[])
 }
 
 
+void
+array_uint24_be_set (unsigned char data[], const unsigned int input)
+{
+	data[0] = (input >> 16) & 0xFF;
+	data[1] = (input >>  8) & 0xFF;
+	data[2] = input & 0xFF;
+}
+
+
 unsigned int
 array_uint24_le (const unsigned char data[])
 {
