@@ -22,24 +22,20 @@
 #ifndef ATOMICS_COBALT_H
 #define ATOMICS_COBALT_H
 
-#include "device.h"
-#include "parser.h"
+#include <libdivecomputer/context.h>
+#include <libdivecomputer/device.h>
+#include <libdivecomputer/parser.h>
+#include <libdivecomputer/atomics_cobalt.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
-device_status_t
-atomics_cobalt_device_open (device_t **device);
+dc_status_t
+atomics_cobalt_device_open (dc_device_t **device, dc_context_t *context);
 
-device_status_t
-atomics_cobalt_device_set_simulation (device_t *abstract, unsigned int simulation);
-
-parser_status_t
-atomics_cobalt_parser_create (parser_t **parser);
-
-parser_status_t
-atomics_cobalt_parser_set_calibration (parser_t *abstract, double atmospheric, double hydrostatic);
+dc_status_t
+atomics_cobalt_parser_create (dc_parser_t **parser, dc_context_t *context);
 
 #ifdef __cplusplus
 }
