@@ -21,9 +21,9 @@
 
 #include <stdlib.h>
 
-#include <libdivecomputer/atomics_cobalt.h>
 #include <libdivecomputer/units.h>
 
+#include "atomics_cobalt.h"
 #include "context-private.h"
 #include "parser-private.h"
 #include "array.h"
@@ -48,34 +48,33 @@ static dc_status_t atomics_cobalt_parser_set_data (dc_parser_t *abstract, const
 static dc_status_t atomics_cobalt_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime);
 static dc_status_t atomics_cobalt_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value);
 static dc_status_t atomics_cobalt_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
-static dc_status_t atomics_cobalt_parser_destroy (dc_parser_t *abstract);
 
 static const dc_parser_vtable_t atomics_cobalt_parser_vtable = {
+	sizeof(atomics_cobalt_parser_t),
 	DC_FAMILY_ATOMICS_COBALT,
 	atomics_cobalt_parser_set_data, /* set_data */
 	atomics_cobalt_parser_get_datetime, /* datetime */
 	atomics_cobalt_parser_get_field, /* fields */
 	atomics_cobalt_parser_samples_foreach, /* samples_foreach */
-	atomics_cobalt_parser_destroy /* destroy */
+	NULL /* destroy */
 };
 
 
 dc_status_t
 atomics_cobalt_parser_create (dc_parser_t **out, dc_context_t *context)
 {
+	atomics_cobalt_parser_t *parser = NULL;
+
 	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
-	atomics_cobalt_parser_t *parser = (atomics_cobalt_parser_t *) malloc (sizeof (atomics_cobalt_parser_t));
+	parser = (atomics_cobalt_parser_t *) dc_parser_allocate (context, &atomics_cobalt_parser_vtable);
 	if (parser == NULL) {
 		ERROR (context, "Failed to allocate memory.");
 		return DC_STATUS_NOMEMORY;
 	}
 
-	// Initialize the base class.
-	parser_init (&parser->base, context, &atomics_cobalt_parser_vtable);
-
 	// Set the default values.
 	parser->atmospheric = 0.0;
 	parser->hydrostatic = 1025.0 * GRAVITY;
@@ -86,16 +85,6 @@ atomics_cobalt_parser_create (dc_parser_t **out, dc_context_t *context)
 }
 
 
-static dc_status_t
-atomics_cobalt_parser_destroy (dc_parser_t *abstract)
-{
-	// Free memory.
-	free (abstract);
-
-	return DC_STATUS_SUCCESS;
-}
-
-
 static dc_status_t
 atomics_cobalt_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size)
 {
@@ -133,6 +122,7 @@ atomics_cobalt_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *dateti
 		datetime->hour   = p[0x18];
 		datetime->minute = p[0x19];
 		datetime->second = 0;
+		datetime->timezone = DC_TIMEZONE_NONE;
 	}
 
 	return DC_STATUS_SUCCESS;
@@ -150,6 +140,7 @@ atomics_cobalt_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, un
 	const unsigned char *p = abstract->data;
 
 	dc_gasmix_t *gasmix = (dc_gasmix_t *) value;
+	dc_tank_t *tank = (dc_tank_t *) value;
 
 	double atmospheric = 0.0;
 	if (parser->atmospheric)
@@ -157,6 +148,8 @@ atomics_cobalt_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, un
 	else
 		atmospheric = array_uint16_le (p + 0x26) * BAR / 1000.0;
 
+	unsigned int workpressure = 0;
+
 	if (value) {
 		switch (type) {
 		case DC_FIELD_DIVETIME:
@@ -166,6 +159,7 @@ atomics_cobalt_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, un
 			*((double *) value) = (array_uint16_le (p + 0x56) * BAR / 1000.0 - atmospheric) / parser->hydrostatic;
 			break;
 		case DC_FIELD_GASMIX_COUNT:
+		case DC_FIELD_TANK_COUNT:
 			*((unsigned int *) value) = p[0x2a];
 			break;
 		case DC_FIELD_GASMIX:
@@ -173,6 +167,47 @@ atomics_cobalt_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, un
 			gasmix->oxygen = p[SZ_HEADER + SZ_GASMIX * flags + 4] / 100.0;
 			gasmix->nitrogen = 1.0 - gasmix->oxygen - gasmix->helium;
 			break;
+		case DC_FIELD_TEMPERATURE_SURFACE:
+			*((double *) value) = (p[0x1B] - 32.0) * (5.0 / 9.0);
+			break;
+		case DC_FIELD_TANK:
+			p += SZ_HEADER + SZ_GASMIX * flags;
+			switch (p[2]) {
+			case 1: // Cuft at psi
+			case 2: // Cuft at bar
+				workpressure = array_uint16_le(p + 10);
+				if (workpressure == 0)
+					return DC_STATUS_DATAFORMAT;
+				tank->type = DC_TANKVOLUME_IMPERIAL;
+				tank->volume = array_uint16_le(p + 8) * CUFT * 1000.0;
+				tank->volume /= workpressure * PSI / ATM;
+				tank->workpressure = workpressure * PSI / BAR;
+				break;
+			case 3: // Wet volume in 1/10 liter
+				tank->type = DC_TANKVOLUME_METRIC;
+				tank->volume = array_uint16_le(p + 8) / 10.0;
+				tank->workpressure = 0.0;
+				break;
+			default:
+				return DC_STATUS_DATAFORMAT;
+			}
+			tank->gasmix = flags;
+			tank->beginpressure = array_uint16_le(p + 6) * PSI / BAR;
+			tank->endpressure = array_uint16_le(p + 14) * PSI / BAR;
+			break;
+		case DC_FIELD_DIVEMODE:
+			switch(p[0x24]) {
+			case 0: // Open Circuit Trimix
+			case 2: // Open Circuit Nitrox
+				*((dc_divemode_t *) value) = DC_DIVEMODE_OC;
+				break;
+			case 1: // Closed Circuit
+				*((dc_divemode_t *) value) = DC_DIVEMODE_CC;
+				break;
+			default:
+				return DC_STATUS_DATAFORMAT;
+			}
+			break;
 		default:
 			return DC_STATUS_UNSUPPORTED;
 		}
@@ -213,6 +248,19 @@ atomics_cobalt_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback
 	// Previous gas mix - initialize with impossible value
 	unsigned int gasmix_previous = 0xFFFFFFFF;
 
+	// Get the primary tank.
+	unsigned int tank = 0;
+	while (tank < ngasmixes) {
+		unsigned int sensor = array_uint16_le(data + SZ_HEADER + SZ_GASMIX * tank + 12);
+		if (sensor == 1)
+			break;
+		tank++;
+	}
+	if (tank >= ngasmixes) {
+		ERROR (abstract->context, "Invalid primary tank index.");
+		return DC_STATUS_DATAFORMAT;
+	}
+
 	unsigned int time = 0;
 	unsigned int in_deco = 0;
 	unsigned int offset = header;
@@ -231,23 +279,25 @@ atomics_cobalt_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback
 
 		// Pressure (1 psi).
 		unsigned int pressure = array_uint16_le (data + offset + 2);
-		sample.pressure.tank = 0;
+		sample.pressure.tank = tank;
 		sample.pressure.value = pressure * PSI / BAR;
 		if (callback) callback (DC_SAMPLE_PRESSURE, sample, userdata);
 
 		// Current gas mix
 		unsigned int gasmix = data[offset + 4];
-		if (gasmix >= ngasmixes) {
-			return DC_STATUS_DATAFORMAT;
-		}
 		if (gasmix != gasmix_previous) {
-			unsigned int o2 = data[SZ_HEADER + SZ_GASMIX * gasmix + 4];
-			unsigned int he = data[SZ_HEADER + SZ_GASMIX * gasmix + 5];
-			sample.event.type = SAMPLE_EVENT_GASCHANGE2;
-			sample.event.time = 0;
-			sample.event.flags = 0;
-			sample.event.value = o2 | (he << 16);
-			if (callback) callback (DC_SAMPLE_EVENT, sample, userdata);
+			unsigned int idx = 0;
+			while (idx < ngasmixes) {
+				if (data[SZ_HEADER + SZ_GASMIX * idx + 0] == gasmix)
+					break;
+				idx++;
+			}
+			if (idx >= ngasmixes) {
+				ERROR (abstract->context, "Invalid gas mix index.");
+				return DC_STATUS_DATAFORMAT;
+			}
+			sample.gasmix = idx;
+			if (callback) callback (DC_SAMPLE_GASMIX, sample, userdata);
 			gasmix_previous = gasmix;
 		}
 
