@@ -22,21 +22,19 @@
 #ifndef CRESSI_EDY_H
 #define CRESSI_EDY_H
 
-#include "device.h"
-#include "parser.h"
+#include <libdivecomputer/context.h>
+#include <libdivecomputer/device.h>
+#include <libdivecomputer/parser.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
-#define CRESSI_EDY_MEMORY_SIZE 0x8000
-#define CRESSI_EDY_PACKET_SIZE 128
+dc_status_t
+cressi_edy_device_open (dc_device_t **device, dc_context_t *context, const char *name);
 
-device_status_t
-cressi_edy_device_open (device_t **device, const char* name);
-
-parser_status_t
-cressi_edy_parser_create (parser_t **parser, unsigned int model);
+dc_status_t
+cressi_edy_parser_create (dc_parser_t **parser, dc_context_t *context, unsigned int model);
 
 #ifdef __cplusplus
 }
