@@ -1,18 +1,18 @@
-/* 
+/*
  * libdivecomputer
- * 
+ *
  * Copyright (C) 2008 Jef Driesen
- * 
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
@@ -23,24 +23,59 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <libdivecomputer/suunto.h>
-#include <libdivecomputer/reefnet.h>
-#include <libdivecomputer/uwatec.h>
-#include <libdivecomputer/oceanic.h>
-#include <libdivecomputer/mares.h>
-#include <libdivecomputer/hw.h>
-#include <libdivecomputer/cressi.h>
-#include <libdivecomputer/zeagle.h>
-#include <libdivecomputer/atomics.h>
-#include <libdivecomputer/shearwater.h>
+#include "suunto_d9.h"
+#include "suunto_eon.h"
+#include "suunto_eonsteel.h"
+#include "suunto_solution.h"
+#include "suunto_vyper2.h"
+#include "suunto_vyper.h"
+#include "reefnet_sensus.h"
+#include "reefnet_sensuspro.h"
+#include "reefnet_sensusultra.h"
+#include "uwatec_aladin.h"
+#include "uwatec_g2.h"
+#include "uwatec_memomouse.h"
+#include "uwatec_meridian.h"
+#include "uwatec_smart.h"
+#include "oceanic_atom2.h"
+#include "oceanic_veo250.h"
+#include "oceanic_vtpro.h"
+#include "mares_darwin.h"
+#include "mares_iconhd.h"
+#include "mares_nemo.h"
+#include "mares_puck.h"
+#include "hw_frog.h"
+#include "hw_ostc.h"
+#include "hw_ostc3.h"
+#include "cressi_edy.h"
+#include "cressi_leonardo.h"
+#include "zeagle_n2ition3.h"
+#include "atomics_cobalt.h"
+#include "shearwater_petrel.h"
+#include "shearwater_predator.h"
+#include "diverite_nitekq.h"
+#include "citizen_aqualand.h"
+#include "divesystem_idive.h"
+#include "cochran_commander.h"
 
 #include "device-private.h"
 #include "context-private.h"
 
-
-void
-device_init (dc_device_t *device, dc_context_t *context, const dc_device_vtable_t *vtable)
+dc_device_t *
+dc_device_allocate (dc_context_t *context, const dc_device_vtable_t *vtable)
 {
+	dc_device_t *device = NULL;
+
+	assert(vtable != NULL);
+	assert(vtable->size >= sizeof(dc_device_t));
+
+	// Allocate memory.
+	device = (dc_device_t *) malloc (vtable->size);
+	if (device == NULL) {
+		ERROR (context, "Failed to allocate memory.");
+		return device;
+	}
+
 	device->vtable = vtable;
 
 	device->context = context;
@@ -54,6 +89,14 @@ device_init (dc_device_t *device, dc_context_t *context, const dc_device_vtable_
 
 	memset (&device->devinfo, 0, sizeof (device->devinfo));
 	memset (&device->clock, 0, sizeof (device->clock));
+
+	return device;
+}
+
+void
+dc_device_deallocate (dc_device_t *device)
+{
+	free (device);
 }
 
 dc_status_t
@@ -81,6 +124,9 @@ dc_device_open (dc_device_t **out, dc_context_t *context, dc_descriptor_t *descr
 	case DC_FAMILY_SUUNTO_D9:
 		rc = suunto_d9_device_open (&device, context, name, dc_descriptor_get_model (descriptor));
 		break;
+	case DC_FAMILY_SUUNTO_EONSTEEL:
+		rc = suunto_eonsteel_device_open (&device, context, dc_descriptor_get_model (descriptor));
+		break;
 	case DC_FAMILY_UWATEC_ALADIN:
 		rc = uwatec_aladin_device_open (&device, context, name);
 		break;
@@ -90,6 +136,12 @@ dc_device_open (dc_device_t **out, dc_context_t *context, dc_descriptor_t *descr
 	case DC_FAMILY_UWATEC_SMART:
 		rc = uwatec_smart_device_open (&device, context);
 		break;
+	case DC_FAMILY_UWATEC_MERIDIAN:
+		rc = uwatec_meridian_device_open (&device, context, name);
+		break;
+	case DC_FAMILY_UWATEC_G2:
+		rc = uwatec_g2_device_open (&device, context, dc_descriptor_get_model (descriptor));
+		break;
 	case DC_FAMILY_REEFNET_SENSUS:
 		rc = reefnet_sensus_device_open (&device, context, name);
 		break;
@@ -100,13 +152,13 @@ dc_device_open (dc_device_t **out, dc_context_t *context, dc_descriptor_t *descr
 		rc = reefnet_sensusultra_device_open (&device, context, name);
 		break;
 	case DC_FAMILY_OCEANIC_VTPRO:
-		rc = oceanic_vtpro_device_open (&device, context, name);
+		rc = oceanic_vtpro_device_open (&device, context, name, dc_descriptor_get_model (descriptor));
 		break;
 	case DC_FAMILY_OCEANIC_VEO250:
 		rc = oceanic_veo250_device_open (&device, context, name);
 		break;
 	case DC_FAMILY_OCEANIC_ATOM2:
-		rc = oceanic_atom2_device_open (&device, context, name);
+		rc = oceanic_atom2_device_open (&device, context, name, dc_descriptor_get_model (descriptor));
 		break;
 	case DC_FAMILY_MARES_NEMO:
 		rc = mares_nemo_device_open (&device, context, name);
@@ -118,7 +170,7 @@ dc_device_open (dc_device_t **out, dc_context_t *context, dc_descriptor_t *descr
 		rc = mares_darwin_device_open (&device, context, name, dc_descriptor_get_model (descriptor));
 		break;
 	case DC_FAMILY_MARES_ICONHD:
-		rc = mares_iconhd_device_open (&device, context, name, dc_descriptor_get_model (descriptor));
+		rc = mares_iconhd_device_open (&device, context, name);
 		break;
 	case DC_FAMILY_HW_OSTC:
 		rc = hw_ostc_device_open (&device, context, name);
@@ -147,6 +199,18 @@ dc_device_open (dc_device_t **out, dc_context_t *context, dc_descriptor_t *descr
 	case DC_FAMILY_SHEARWATER_PETREL:
 		rc = shearwater_petrel_device_open (&device, context, name);
 		break;
+	case DC_FAMILY_DIVERITE_NITEKQ:
+		rc = diverite_nitekq_device_open (&device, context, name);
+		break;
+	case DC_FAMILY_CITIZEN_AQUALAND:
+		rc = citizen_aqualand_device_open (&device, context, name);
+		break;
+	case DC_FAMILY_DIVESYSTEM_IDIVE:
+		rc = divesystem_idive_device_open (&device, context, name, dc_descriptor_get_model (descriptor));
+		break;
+	case DC_FAMILY_COCHRAN_COMMANDER:
+		rc = cochran_commander_device_open (&device, context, name);
+		break;
 	default:
 		return DC_STATUS_INVALIDARGS;
 	}
@@ -307,26 +371,44 @@ dc_device_foreach (dc_device_t *device, dc_dive_callback_t callback, void *userd
 
 
 dc_status_t
-dc_device_close (dc_device_t *device)
+dc_device_timesync (dc_device_t *device, const dc_datetime_t *datetime)
 {
 	if (device == NULL)
-		return DC_STATUS_SUCCESS;
-
-	if (device->vtable->close == NULL)
 		return DC_STATUS_UNSUPPORTED;
 
+	if (device->vtable->timesync == NULL)
+		return DC_STATUS_UNSUPPORTED;
+
+	return device->vtable->timesync (device, datetime);
+}
+
+
+dc_status_t
+dc_device_close (dc_device_t *device)
+{
+	dc_status_t status = DC_STATUS_SUCCESS;
+
+	if (device == NULL)
+		return DC_STATUS_SUCCESS;
+
 	// Disable the cancellation callback.
 	device->cancel_callback = NULL;
 	device->cancel_userdata = NULL;
 
-	return device->vtable->close (device);
+	if (device->vtable->close) {
+		status = device->vtable->close (device);
+	}
+
+	dc_device_deallocate (device);
+
+	return status;
 }
 
 
 void
 device_event_emit (dc_device_t *device, dc_event_type_t event, const void *data)
 {
-	dc_event_progress_t *progress = (dc_event_progress_t *) data;
+	const dc_event_progress_t *progress = (const dc_event_progress_t *) data;
 
 	// Check the event data for errors.
 	switch (event) {
@@ -354,10 +436,10 @@ device_event_emit (dc_device_t *device, dc_event_type_t event, const void *data)
 	// Cache the event data.
 	switch (event) {
 	case DC_EVENT_DEVINFO:
-		device->devinfo = *(dc_event_devinfo_t *) data;
+		device->devinfo = *(const dc_event_devinfo_t *) data;
 		break;
 	case DC_EVENT_CLOCK:
-		device->clock = *(dc_event_clock_t *) data;
+		device->clock = *(const dc_event_clock_t *) data;
 		break;
 	default:
 		break;
