@@ -22,20 +22,19 @@
 #include <string.h> // memcmp, memcpy
 #include <stdlib.h> // malloc, free
 
-#include <libdivecomputer/hw_ostc.h>
-
+#include "hw_ostc.h"
 #include "context-private.h"
 #include "device-private.h"
 #include "serial.h"
 #include "checksum.h"
 #include "array.h"
+#include "ihex.h"
 
 #define ISINSTANCE(device) dc_device_isinstance((device), &hw_ostc_device_vtable)
 
-#define EXITCODE(rc) \
-( \
-	rc == -1 ? DC_STATUS_IO : DC_STATUS_TIMEOUT \
-)
+#define C_ARRAY_SIZE(a) (sizeof(a) / sizeof(*(a)))
+
+#define MAXRETRIES 9
 
 #define FW_190 0x015A
 
@@ -45,6 +44,13 @@
 #define SZ_FW_190 0x8000
 #define SZ_FW_NEW 0x10000
 
+#define SZ_FIRMWARE 0x17F40
+#define SZ_BLOCK    0x40
+
+#define ACK     0x4B /* "K" for ok */
+#define NAK     0x4E /* "N" for not ok */
+#define PICTYPE 0x57 /* PIC type (18F4685) */
+
 #define WIDTH  320
 #define HEIGHT 240
 #define BLACK  0x00
@@ -52,46 +58,57 @@
 
 typedef struct hw_ostc_device_t {
 	dc_device_t base;
-	serial_t *port;
+	dc_serial_t *port;
 	unsigned char fingerprint[5];
 } hw_ostc_device_t;
 
+typedef struct hw_ostc_firmware_t {
+	unsigned char data[SZ_FIRMWARE];
+	unsigned char bitmap[SZ_FIRMWARE / SZ_BLOCK];
+} hw_ostc_firmware_t;
+
 static dc_status_t hw_ostc_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size);
 static dc_status_t hw_ostc_device_dump (dc_device_t *abstract, dc_buffer_t *buffer);
 static dc_status_t hw_ostc_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
+static dc_status_t hw_ostc_device_timesync (dc_device_t *abstract, const dc_datetime_t *datetime);
 static dc_status_t hw_ostc_device_close (dc_device_t *abstract);
 
 static const dc_device_vtable_t hw_ostc_device_vtable = {
+	sizeof(hw_ostc_device_t),
 	DC_FAMILY_HW_OSTC,
 	hw_ostc_device_set_fingerprint, /* set_fingerprint */
 	NULL, /* read */
 	NULL, /* write */
 	hw_ostc_device_dump, /* dump */
 	hw_ostc_device_foreach, /* foreach */
+	hw_ostc_device_timesync, /* timesync */
 	hw_ostc_device_close /* close */
 };
 
+static dc_status_t
+hw_ostc_extract_dives (dc_device_t *device, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata);
 
 static dc_status_t
 hw_ostc_send (hw_ostc_device_t *device, unsigned char cmd, unsigned int echo)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
 	dc_device_t *abstract = (dc_device_t *) device;
 
 	// Send the command.
 	unsigned char command[1] = {cmd};
-	int n = serial_write (device->port, command, sizeof (command));
-	if (n != sizeof (command)) {
+	status = dc_serial_write (device->port, command, sizeof (command), NULL);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
-		return EXITCODE (n);
+		return status;
 	}
 
 	if (echo) {
 		// Read the echo.
 		unsigned char answer[1] = {0};
-		n = serial_read (device->port, answer, sizeof (answer));
-		if (n != sizeof (answer)) {
+		status = dc_serial_read (device->port, answer, sizeof (answer), NULL);
+		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive the echo.");
-			return EXITCODE (n);
+			return status;
 		}
 
 		// Verify the echo.
@@ -108,73 +125,74 @@ hw_ostc_send (hw_ostc_device_t *device, unsigned char cmd, unsigned int echo)
 dc_status_t
 hw_ostc_device_open (dc_device_t **out, dc_context_t *context, const char *name)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
+	hw_ostc_device_t *device = NULL;
+
 	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
-	hw_ostc_device_t *device = (hw_ostc_device_t *) malloc (sizeof (hw_ostc_device_t));
+	device = (hw_ostc_device_t *) dc_device_allocate (context, &hw_ostc_device_vtable);
 	if (device == NULL) {
 		ERROR (context, "Failed to allocate memory.");
 		return DC_STATUS_NOMEMORY;
 	}
 
-	// Initialize the base class.
-	device_init (&device->base, context, &hw_ostc_device_vtable);
-
 	// Set the default values.
 	device->port = NULL;
 	memset (device->fingerprint, 0, sizeof (device->fingerprint));
 
 	// Open the device.
-	int rc = serial_open (&device->port, context, name);
-	if (rc == -1) {
+	status = dc_serial_open (&device->port, context, name);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to open the serial port.");
-		free (device);
-		return DC_STATUS_IO;
+		goto error_free;
 	}
 
 	// Set the serial communication protocol (115200 8N1).
-	rc = serial_configure (device->port, 115200, 8, SERIAL_PARITY_NONE, 1, SERIAL_FLOWCONTROL_NONE);
-	if (rc == -1) {
+	status = dc_serial_configure (device->port, 115200, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the terminal attributes.");
-		serial_close (device->port);
-		free (device);
-		return DC_STATUS_IO;
+		goto error_close;
 	}
 
 	// Set the timeout for receiving data.
-	if (serial_set_timeout (device->port, 4000) == -1) {
+	status = dc_serial_set_timeout (device->port, 4000);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
-		serial_close (device->port);
-		free (device);
-		return DC_STATUS_IO;
+		goto error_close;
 	}
 
 	// Make sure everything is in a sane state.
-	serial_sleep (device->port, 100);
-	serial_flush (device->port, SERIAL_QUEUE_BOTH);
+	dc_serial_sleep (device->port, 100);
+	dc_serial_purge (device->port, DC_DIRECTION_ALL);
 
 	*out = (dc_device_t*) device;
 
 	return DC_STATUS_SUCCESS;
+
+error_close:
+	dc_serial_close (device->port);
+error_free:
+	dc_device_deallocate ((dc_device_t *) device);
+	return status;
 }
 
 
 static dc_status_t
 hw_ostc_device_close (dc_device_t *abstract)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
 	hw_ostc_device_t *device = (hw_ostc_device_t*) abstract;
+	dc_status_t rc = DC_STATUS_SUCCESS;
 
 	// Close the device.
-	if (serial_close (device->port) == -1) {
-		free (device);
-		return DC_STATUS_IO;
+	rc = dc_serial_close (device->port);
+	if (rc != DC_STATUS_SUCCESS) {
+		dc_status_set_error(&status, rc);
 	}
 
-	// Free memory.
-	free (device);
-
-	return DC_STATUS_SUCCESS;
+	return status;
 }
 
 
@@ -198,6 +216,7 @@ hw_ostc_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[
 static dc_status_t
 hw_ostc_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
 	hw_ostc_device_t *device = (hw_ostc_device_t*) abstract;
 
 	// Erase the current contents of the buffer.
@@ -213,18 +232,18 @@ hw_ostc_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 
 	// Send the command.
 	unsigned char command[1] = {'a'};
-	int rc = serial_write (device->port, command, sizeof (command));
-	if (rc != sizeof (command)) {
+	status = dc_serial_write (device->port, command, sizeof (command), NULL);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
-		return EXITCODE (rc);
+		return status;
 	}
 
 	// Read the header.
 	unsigned char header[SZ_HEADER] = {0};
-	int n = serial_read (device->port, header, sizeof (header));
-	if (n != sizeof (header)) {
+	status = dc_serial_read (device->port, header, sizeof (header), NULL);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the header.");
-		return EXITCODE (n);
+		return status;
 	}
 
 	// Verify the header.
@@ -266,8 +285,9 @@ hw_ostc_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 		unsigned int len = 1024;
 
 		// Increase the packet size if more data is immediately available.
-		int available = serial_get_received (device->port);
-		if (available > len)
+		size_t available = 0;
+		status = dc_serial_get_available (device->port, &available);
+		if (status == DC_STATUS_SUCCESS && available > len)
 			len = available;
 
 		// Limit the packet size to the total size.
@@ -275,10 +295,10 @@ hw_ostc_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 			len = size - nbytes;
 
 		// Read the packet.
-		int n = serial_read (device->port, data + nbytes, len);
-		if (n != len) {
+		status = dc_serial_read (device->port, data + nbytes, len, NULL);
+		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive the answer.");
-			return EXITCODE (n);
+			return status;
 		}
 
 		// Update and emit a progress event.
@@ -310,7 +330,9 @@ hw_ostc_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void
 	dc_event_devinfo_t devinfo;
 	devinfo.firmware = array_uint16_be (data + 264);
 	devinfo.serial = array_uint16_le (data + 6);
-	if (devinfo.serial > 2048)
+	if (devinfo.serial > 7000)
+		devinfo.model = 3; // OSTC 2C
+	else if (devinfo.serial > 2048)
 		devinfo.model = 2; // OSTC 2N
 	else if (devinfo.serial > 300)
 		devinfo.model = 1; // OSTC Mk2
@@ -330,6 +352,7 @@ hw_ostc_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void
 dc_status_t
 hw_ostc_device_md2hash (dc_device_t *abstract, unsigned char data[], unsigned int size)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
 	hw_ostc_device_t *device = (hw_ostc_device_t *) abstract;
 
 	if (!ISINSTANCE (abstract))
@@ -346,24 +369,22 @@ hw_ostc_device_md2hash (dc_device_t *abstract, unsigned char data[], unsigned in
 		return rc;
 
 	// Read the answer.
-	int n = serial_read (device->port, data, SZ_MD2HASH);
-	if (n != SZ_MD2HASH) {
+	status = dc_serial_read (device->port, data, SZ_MD2HASH, NULL);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
-		return EXITCODE (n);
+		return status;
 	}
 
 	return DC_STATUS_SUCCESS;
 }
 
 
-dc_status_t
-hw_ostc_device_clock (dc_device_t *abstract, const dc_datetime_t *datetime)
+static dc_status_t
+hw_ostc_device_timesync (dc_device_t *abstract, const dc_datetime_t *datetime)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
 	hw_ostc_device_t *device = (hw_ostc_device_t *) abstract;
 
-	if (!ISINSTANCE (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	if (datetime == NULL) {
 		ERROR (abstract->context, "Invalid parameter specified.");
 		return DC_STATUS_INVALIDARGS;
@@ -378,10 +399,10 @@ hw_ostc_device_clock (dc_device_t *abstract, const dc_datetime_t *datetime)
 	unsigned char packet[6] = {
 		datetime->hour, datetime->minute, datetime->second,
 		datetime->month, datetime->day, datetime->year - 2000};
-	int n = serial_write (device->port, packet, sizeof (packet));
-	if (n != sizeof (packet)) {
+	status = dc_serial_write (device->port, packet, sizeof (packet), NULL);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the data packet.");
-		return EXITCODE (n);
+		return status;
 	}
 
 	return DC_STATUS_SUCCESS;
@@ -391,6 +412,7 @@ hw_ostc_device_clock (dc_device_t *abstract, const dc_datetime_t *datetime)
 dc_status_t
 hw_ostc_device_eeprom_read (dc_device_t *abstract, unsigned int bank, unsigned char data[], unsigned int size)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
 	hw_ostc_device_t *device = (hw_ostc_device_t *) abstract;
 
 	if (!ISINSTANCE (abstract))
@@ -413,10 +435,10 @@ hw_ostc_device_eeprom_read (dc_device_t *abstract, unsigned int bank, unsigned c
 		return rc;
 
 	// Read the answer.
-	int n = serial_read (device->port, data, SZ_EEPROM);
-	if (n != SZ_EEPROM) {
+	status = dc_serial_read (device->port, data, SZ_EEPROM, NULL);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
-		return EXITCODE (n);
+		return status;
 	}
 
 	return DC_STATUS_SUCCESS;
@@ -478,6 +500,7 @@ hw_ostc_device_reset (dc_device_t *abstract)
 dc_status_t
 hw_ostc_device_screenshot (dc_device_t *abstract, dc_buffer_t *buffer, hw_ostc_format_t format)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
 	hw_ostc_device_t *device = (hw_ostc_device_t *) abstract;
 
 	if (!ISINSTANCE (abstract))
@@ -533,13 +556,13 @@ hw_ostc_device_screenshot (dc_device_t *abstract, dc_buffer_t *buffer, hw_ostc_f
 	unsigned int npixels = 0;
 	while (npixels < WIDTH * HEIGHT) {
 		unsigned char raw[3] = {0};
-		int n = serial_read (device->port, raw, 1);
-		if (n != 1) {
+		status = dc_serial_read (device->port, raw, 1, NULL);
+		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive the packet.");
-			return EXITCODE (n);
+			return status;
 		}
 
-		unsigned int nbytes = n;
+		unsigned int nbytes = 1;
 		unsigned int count = raw[0];
 		if ((count & 0x80) == 0x00) {
 			// Black pixel.
@@ -551,13 +574,13 @@ hw_ostc_device_screenshot (dc_device_t *abstract, dc_buffer_t *buffer, hw_ostc_f
 			count &= 0x3F;
 		} else {
 			// Color pixel.
-			n = serial_read (device->port, raw + 1, 2);
-			if (n != 2) {
+			status = dc_serial_read (device->port, raw + 1, 2, NULL);
+			if (status != DC_STATUS_SUCCESS) {
 				ERROR (abstract->context, "Failed to receive the packet.");
-				return EXITCODE (n);
+				return status;
 			}
 
-			nbytes += n;
+			nbytes += 2;
 			count &= 0x3F;
 		}
 		count++;
@@ -610,7 +633,7 @@ hw_ostc_device_screenshot (dc_device_t *abstract, dc_buffer_t *buffer, hw_ostc_f
 }
 
 
-dc_status_t
+static dc_status_t
 hw_ostc_extract_dives (dc_device_t *abstract, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata)
 {
 	hw_ostc_device_t *device = (hw_ostc_device_t *) abstract;
@@ -652,3 +675,299 @@ hw_ostc_extract_dives (dc_device_t *abstract, const unsigned char data[], unsign
 
 	return DC_STATUS_SUCCESS;
 }
+
+
+static dc_status_t
+hw_ostc_firmware_readfile (hw_ostc_firmware_t *firmware, dc_context_t *context, const char *filename)
+{
+	dc_status_t rc = DC_STATUS_SUCCESS;
+
+	if (firmware == NULL) {
+		ERROR (context, "Invalid arguments.");
+		return DC_STATUS_INVALIDARGS;
+	}
+
+	// Initialize the buffers.
+	memset (firmware->data, 0xFF, sizeof (firmware->data));
+	memset (firmware->bitmap, 0x00, sizeof (firmware->bitmap));
+
+	// Open the hex file.
+	dc_ihex_file_t *file = NULL;
+	rc = dc_ihex_file_open (&file, context, filename);
+	if (rc != DC_STATUS_SUCCESS) {
+		ERROR (context, "Failed to open the hex file.");
+		return rc;
+	}
+
+	// Read the hex file.
+	unsigned int lba = 0;
+	dc_ihex_entry_t entry;
+	while ((rc = dc_ihex_file_read (file, &entry)) == DC_STATUS_SUCCESS) {
+		if (entry.type == 0) {
+			// Data record.
+			unsigned int address = (lba << 16) + entry.address;
+			if (address + entry.length > SZ_FIRMWARE) {
+				WARNING (context, "Ignoring out of range record (0x%08x,%u).", address, entry.length);
+				continue;
+			}
+
+			// Copy the record to the buffer.
+			memcpy (firmware->data + address, entry.data, entry.length);
+
+			// Mark the corresponding blocks in the bitmap.
+			unsigned int begin = address / SZ_BLOCK;
+			unsigned int end = (address + entry.length + SZ_BLOCK - 1) / SZ_BLOCK;
+			for (unsigned int i = begin; i < end; ++i) {
+				firmware->bitmap[i] = 1;
+			}
+		} else if (entry.type == 1) {
+			// End of file record.
+			break;
+		} else if (entry.type == 4) {
+			// Extended linear address record.
+			lba = array_uint16_be (entry.data);
+		} else {
+			ERROR (context, "Unexpected record type.");
+			dc_ihex_file_close (file);
+			return DC_STATUS_DATAFORMAT;
+		}
+	}
+	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_DONE) {
+		ERROR (context, "Failed to read the record.");
+		dc_ihex_file_close (file);
+		return rc;
+	}
+
+	// Close the file.
+	dc_ihex_file_close (file);
+
+	// Verify the presence of the first block.
+	if (firmware->bitmap[0] == 0) {
+		ERROR (context, "No first data block.");
+		return DC_STATUS_DATAFORMAT;
+	}
+
+	// Setup the last block.
+	// Copy the "goto main" instruction, stored in the first 8 bytes of the hex
+	// file, to the end of the last block at address 0x17F38. This last block
+	// needs to be present, regardless of whether it's included in the hex file
+	// or not!
+	memset (firmware->data + SZ_FIRMWARE - SZ_BLOCK, 0xFF, SZ_BLOCK - 8);
+	memcpy (firmware->data + SZ_FIRMWARE - 8, firmware->data, 8);
+	firmware->bitmap[C_ARRAY_SIZE(firmware->bitmap) - 1] = 1;
+
+	// Setup the first block.
+	// Copy the hardcoded "goto 0x17F40" instruction to the start of the first
+	// block at address 0x00000.
+	const unsigned char header[] = {0xA0, 0xEF, 0xBF, 0xF0};
+	memcpy (firmware->data, header, sizeof (header));
+
+	return rc;
+}
+
+
+static dc_status_t
+hw_ostc_firmware_setup_internal (hw_ostc_device_t *device)
+{
+	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_device_t *abstract = (dc_device_t *) device;
+
+	// Send the command.
+	unsigned char command[1] = {0xC1};
+	status = dc_serial_write (device->port, command, sizeof (command), NULL);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to send the command.");
+		return status;
+	}
+
+	// Read the response.
+	unsigned char answer[2] = {0};
+	status = dc_serial_read (device->port, answer, sizeof (answer), NULL);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to receive the response.");
+		return status;
+	}
+
+	// Verify the response.
+	const unsigned char expected[2] = {PICTYPE, ACK};
+	if (memcmp (answer, expected, sizeof (expected)) != 0) {
+		ERROR (abstract->context, "Unexpected response.");
+		return DC_STATUS_PROTOCOL;
+	}
+
+	return DC_STATUS_SUCCESS;
+}
+
+
+static dc_status_t
+hw_ostc_firmware_setup (hw_ostc_device_t *device, unsigned int maxretries)
+{
+	dc_status_t rc = DC_STATUS_SUCCESS;
+
+	unsigned int nretries = 0;
+	while ((rc = hw_ostc_firmware_setup_internal (device)) != DC_STATUS_SUCCESS) {
+		if (rc != DC_STATUS_TIMEOUT && rc != DC_STATUS_PROTOCOL)
+			break;
+
+		// Abort if the maximum number of retries is reached.
+		if (nretries++ >= maxretries)
+			break;
+	}
+
+	return rc;
+}
+
+
+static dc_status_t
+hw_ostc_firmware_write_internal (hw_ostc_device_t *device, unsigned char *data, unsigned int size)
+{
+	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_device_t *abstract = (dc_device_t *) device;
+
+	// Send the packet.
+	status = dc_serial_write (device->port, data, size, NULL);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to send the packet.");
+		return status;
+	}
+
+	// Read the response.
+	unsigned char answer[1] = {0};
+	status = dc_serial_read (device->port, answer, sizeof (answer), NULL);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to receive the response.");
+		return status;
+	}
+
+	// Verify the response.
+	const unsigned char expected[] = {ACK};
+	if (memcmp (answer, expected, sizeof (expected)) != 0) {
+		ERROR (abstract->context, "Unexpected response.");
+		return DC_STATUS_PROTOCOL;
+	}
+
+	return DC_STATUS_SUCCESS;
+}
+
+
+static dc_status_t
+hw_ostc_firmware_write (hw_ostc_device_t *device, unsigned char *data, unsigned int size)
+{
+	dc_status_t rc = DC_STATUS_SUCCESS;
+
+	unsigned int nretries = 0;
+	while ((rc = hw_ostc_firmware_write_internal (device, data, size)) != DC_STATUS_SUCCESS) {
+		if (rc != DC_STATUS_TIMEOUT && rc != DC_STATUS_PROTOCOL)
+			break;
+
+		// Abort if the maximum number of retries is reached.
+		if (nretries++ >= MAXRETRIES)
+			break;
+	}
+
+	return rc;
+}
+
+/*
+ * Think twice before modifying the code for updating the ostc firmware!
+ * It has been carefully developed and tested with assistance from
+ * Heinrichs-Weikamp, using a special development unit. If you start
+ * experimenting with a normal unit and accidentally screw up, you might
+ * brick the device permanently and turn it into an expensive
+ * paperweight. You have been warned!
+ */
+dc_status_t
+hw_ostc_device_fwupdate (dc_device_t *abstract, const char *filename)
+{
+	dc_status_t rc = DC_STATUS_SUCCESS;
+	hw_ostc_device_t *device = (hw_ostc_device_t *) abstract;
+	dc_context_t *context = (abstract ? abstract->context : NULL);
+
+	if (!ISINSTANCE (abstract))
+		return DC_STATUS_INVALIDARGS;
+
+	// Allocate memory for the firmware data.
+	hw_ostc_firmware_t *firmware = (hw_ostc_firmware_t *) malloc (sizeof (hw_ostc_firmware_t));
+	if (firmware == NULL) {
+		ERROR (context, "Failed to allocate memory.");
+		return DC_STATUS_NOMEMORY;
+	}
+
+	// Read the hex file.
+	rc = hw_ostc_firmware_readfile (firmware, context, filename);
+	if (rc != DC_STATUS_SUCCESS) {
+		ERROR (context, "Failed to read the firmware file.");
+		free (firmware);
+		return rc;
+	}
+
+	// Temporary set a relative short timeout. The command to setup the
+	// bootloader needs to be send repeatedly, until the response packet is
+	// received. Thus the time between each two attempts is directly controlled
+	// by the timeout value.
+	dc_serial_set_timeout (device->port, 300);
+
+	// Setup the bootloader.
+	const unsigned int baudrates[] = {19200, 115200};
+	for (unsigned int i = 0; i < C_ARRAY_SIZE(baudrates); ++i) {
+		// Adjust the baudrate.
+		rc = dc_serial_configure (device->port, baudrates[i], 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
+		if (rc != DC_STATUS_SUCCESS) {
+			ERROR (abstract->context, "Failed to set the terminal attributes.");
+			free (firmware);
+			return rc;
+		}
+
+		// Try to setup the bootloader.
+		unsigned int maxretries = (i == 0 ? 1 : MAXRETRIES);
+		rc = hw_ostc_firmware_setup (device, maxretries);
+		if (rc == DC_STATUS_SUCCESS)
+			break;
+	}
+	if (rc != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to setup the bootloader.");
+		free (firmware);
+		return rc;
+	}
+
+	// Increase the timeout again.
+	dc_serial_set_timeout (device->port, 1000);
+
+	// Enable progress notifications.
+	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
+	progress.maximum = C_ARRAY_SIZE(firmware->bitmap);
+	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
+
+	for (unsigned int i = 0; i < C_ARRAY_SIZE(firmware->bitmap); ++i) {
+		// Skip empty blocks.
+		if (firmware->bitmap[i] == 0)
+			continue;
+
+		// Create the packet.
+		unsigned int address = i * SZ_BLOCK;
+		unsigned char packet[4 + SZ_BLOCK + 1] = {
+			(address >> 16) & 0xFF,
+			(address >>  8) & 0xFF,
+			(address      ) & 0xFF,
+			SZ_BLOCK
+		};
+		memcpy (packet + 4, firmware->data + address, SZ_BLOCK);
+		packet[sizeof (packet) - 1] = ~checksum_add_uint8 (packet, 4 + SZ_BLOCK, 0x00) + 1;
+
+		// Send the packet.
+		rc = hw_ostc_firmware_write (device, packet, sizeof (packet));
+		if (rc != DC_STATUS_SUCCESS) {
+			ERROR (abstract->context, "Failed to send the packet.");
+			free (firmware);
+			return rc;
+		}
+
+		// Update and emit a progress event.
+		progress.current = i + 1;
+		device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
+	}
+
+	free (firmware);
+
+	return DC_STATUS_SUCCESS;
+}
