@@ -22,21 +22,19 @@
 #ifndef MARES_DARWIN_H
 #define MARES_DARWIN_H
 
-#include "device.h"
-#include "parser.h"
+#include <libdivecomputer/context.h>
+#include <libdivecomputer/device.h>
+#include <libdivecomputer/parser.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
-device_status_t
-mares_darwin_device_open (device_t **device, const char *name, unsigned int model);
+dc_status_t
+mares_darwin_device_open (dc_device_t **device, dc_context_t *context, const char *name, unsigned int model);
 
-device_status_t
-mares_darwin_extract_dives (device_t *device, const unsigned char data[], unsigned int size, dive_callback_t callback, void *userdata);
-
-parser_status_t
-mares_darwin_parser_create (parser_t **parser, unsigned int model);
+dc_status_t
+mares_darwin_parser_create (dc_parser_t **parser, dc_context_t *context, unsigned int model);
 
 #ifdef __cplusplus
 }
