@@ -22,23 +22,19 @@
 #ifndef MARES_ICONHD_H
 #define MARES_ICONHD_H
 
-#include "device.h"
-#include "parser.h"
+#include <libdivecomputer/context.h>
+#include <libdivecomputer/device.h>
+#include <libdivecomputer/parser.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
-#define MARES_ICONHD_MEMORY_SIZE 0x100000
+dc_status_t
+mares_iconhd_device_open (dc_device_t **device, dc_context_t *context, const char *name);
 
-device_status_t
-mares_iconhd_device_open (device_t **device, const char* name);
-
-device_status_t
-mares_iconhd_extract_dives (device_t *abstract, const unsigned char data[], unsigned int size, dive_callback_t callback, void *userdata);
-
-parser_status_t
-mares_iconhd_parser_create (parser_t **parser, unsigned int model);
+dc_status_t
+mares_iconhd_parser_create (dc_parser_t **parser, dc_context_t *context, unsigned int model);
 
 #ifdef __cplusplus
 }
