@@ -1,18 +1,18 @@
-/* 
+/*
  * libdivecomputer
- * 
+ *
  * Copyright (C) 2008 Jef Driesen
- * 
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
@@ -22,9 +22,9 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <libdivecomputer/mares_nemo.h>
 #include <libdivecomputer/units.h>
 
+#include "mares_nemo.h"
 #include "context-private.h"
 #include "parser-private.h"
 #include "array.h"
@@ -39,6 +39,11 @@
 #define NEMOAPNEIST 18
 #define PUCKAIR     19
 
+#define AIR      0
+#define NITROX   1
+#define FREEDIVE 2
+#define GAUGE    3
+
 typedef struct mares_nemo_parser_t mares_nemo_parser_t;
 
 struct mares_nemo_parser_t {
@@ -58,43 +63,42 @@ static dc_status_t mares_nemo_parser_set_data (dc_parser_t *abstract, const unsi
 static dc_status_t mares_nemo_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime);
 static dc_status_t mares_nemo_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value);
 static dc_status_t mares_nemo_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
-static dc_status_t mares_nemo_parser_destroy (dc_parser_t *abstract);
 
 static const dc_parser_vtable_t mares_nemo_parser_vtable = {
+	sizeof(mares_nemo_parser_t),
 	DC_FAMILY_MARES_NEMO,
 	mares_nemo_parser_set_data, /* set_data */
 	mares_nemo_parser_get_datetime, /* datetime */
 	mares_nemo_parser_get_field, /* fields */
 	mares_nemo_parser_samples_foreach, /* samples_foreach */
-	mares_nemo_parser_destroy /* destroy */
+	NULL /* destroy */
 };
 
 
 dc_status_t
 mares_nemo_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int model)
 {
+	mares_nemo_parser_t *parser = NULL;
+
 	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
-	mares_nemo_parser_t *parser = (mares_nemo_parser_t *) malloc (sizeof (mares_nemo_parser_t));
+	parser = (mares_nemo_parser_t *) dc_parser_allocate (context, &mares_nemo_parser_vtable);
 	if (parser == NULL) {
 		ERROR (context, "Failed to allocate memory.");
 		return DC_STATUS_NOMEMORY;
 	}
 
-	// Initialize the base class.
-	parser_init (&parser->base, context, &mares_nemo_parser_vtable);
-
 	// Get the freedive mode for this model.
-	unsigned int freedive = 2;
-	if (model == NEMOWIDE || model == PUCK || model == PUCKAIR)
-		freedive = 3;
+	unsigned int freedive = FREEDIVE;
+	if (model == NEMOWIDE || model == NEMOAIR || model == PUCK || model == PUCKAIR)
+		freedive = GAUGE;
 
 	// Set the default values.
 	parser->model = model;
 	parser->freedive = freedive;
-	parser->mode = 0;
+	parser->mode = AIR;
 	parser->length = 0;
 	parser->sample_count = 0;
 	parser->sample_size = 0;
@@ -107,16 +111,6 @@ mares_nemo_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int
 }
 
 
-static dc_status_t
-mares_nemo_parser_destroy (dc_parser_t *abstract)
-{
-	// Free memory.
-	free (abstract);
-
-	return DC_STATUS_SUCCESS;
-}
-
-
 static dc_status_t
 mares_nemo_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size)
 {
@@ -125,7 +119,7 @@ mares_nemo_parser_set_data (dc_parser_t *abstract, const unsigned char *data, un
 	// Clear the previous state.
 	parser->base.data = NULL;
 	parser->base.size = 0;
-	parser->mode = 0;
+	parser->mode = AIR;
 	parser->length = 0;
 	parser->sample_count = 0;
 	parser->sample_size = 0;
@@ -206,6 +200,7 @@ mares_nemo_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime)
 		datetime->hour   = p[3];
 		datetime->minute = p[4];
 		datetime->second = 0;
+		datetime->timezone = DC_TIMEZONE_NONE;
 	}
 
 	return DC_STATUS_SUCCESS;
@@ -226,6 +221,7 @@ mares_nemo_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsign
 	if (value) {
 		if (parser->mode != parser->freedive) {
 			dc_gasmix_t *gasmix = (dc_gasmix_t *) value;
+			dc_tank_t *tank = (dc_tank_t *) value;
 			switch (type) {
 			case DC_FIELD_DIVETIME:
 				*((unsigned int *) value) = parser->sample_count * 20;
@@ -234,17 +230,17 @@ mares_nemo_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsign
 				*((double *) value) = array_uint16_le (p + 53 - 10) / 10.0;
 				break;
 			case DC_FIELD_GASMIX_COUNT:
-				if (parser->mode == 0 || parser->mode == 1)
+				if (parser->mode == AIR || parser->mode == NITROX)
 					*((unsigned int *) value) = 1;
 				else
 					*((unsigned int *) value) = 0;
 				break;
 			case DC_FIELD_GASMIX:
 				switch (parser->mode) {
-				case 0: // Air
+				case AIR:
 					gasmix->oxygen = 0.21;
 					break;
-				case 1: // Nitrox
+				case NITROX:
 					gasmix->oxygen = p[53 - 43] / 100.0;
 					break;
 				default:
@@ -253,6 +249,62 @@ mares_nemo_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsign
 				gasmix->helium = 0.0;
 				gasmix->nitrogen = 1.0 - gasmix->oxygen - gasmix->helium;
 				break;
+			case DC_FIELD_TANK_COUNT:
+				if (parser->extra)
+					*((unsigned int *) value) = 1;
+				else
+					*((unsigned int *) value) = 0;
+				break;
+			case DC_FIELD_TANK:
+				if (parser->extra == 12) {
+					unsigned int volume = array_uint16_le(p + parser->header + 0);
+					unsigned int workpressure = array_uint16_le(p + parser->header + 2);
+					if (workpressure == 0xFFFF) {
+						tank->type = DC_TANKVOLUME_METRIC;
+						tank->volume = volume / 10.0;
+						tank->workpressure = 0.0;
+					} else {
+						if (workpressure == 0)
+							return DC_STATUS_DATAFORMAT;
+						tank->type = DC_TANKVOLUME_IMPERIAL;
+						tank->volume = volume * CUFT * 1000.0;
+						tank->volume /= workpressure * PSI / ATM;
+						tank->workpressure = workpressure * PSI / BAR;
+					}
+					tank->beginpressure = array_uint16_le(p + parser->header + 4) / 100.0;
+					tank->endpressure = array_uint16_le(p + parser->header + 6) / 100.0;
+				} else if (parser->extra == 7) {
+					tank->type = DC_TANKVOLUME_NONE;
+					tank->volume = 0.0;
+					tank->workpressure = 0.0;
+					tank->beginpressure = array_uint16_le(p + parser->header + 0);
+					tank->endpressure = array_uint16_le(p + parser->header + 2);
+				} else {
+					return DC_STATUS_UNSUPPORTED;
+				}
+				if (parser->mode == AIR || parser->mode == NITROX) {
+					tank->gasmix = 0;
+				} else {
+					tank->gasmix = DC_GASMIX_UNKNOWN;
+				}
+				break;
+			case DC_FIELD_TEMPERATURE_MINIMUM:
+				*((double *) value) = (signed char) p[53 - 11];
+				break;
+			case DC_FIELD_DIVEMODE:
+				switch (parser->mode) {
+				case AIR:
+				case NITROX:
+					*((dc_divemode_t *) value) = DC_DIVEMODE_OC;
+					break;
+				case FREEDIVE:
+				case GAUGE:
+					*((dc_divemode_t *) value) = DC_DIVEMODE_GAUGE;
+					break;
+				default:
+					return DC_STATUS_DATAFORMAT;
+				}
+				break;
 			default:
 				return DC_STATUS_UNSUPPORTED;
 			}
@@ -272,6 +324,12 @@ mares_nemo_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsign
 			case DC_FIELD_GASMIX_COUNT:
 				*((unsigned int *) value) = 0;
 				break;
+			case DC_FIELD_TEMPERATURE_MINIMUM:
+				*((double *) value) = (signed char) p[28 - 11];
+				break;
+			case DC_FIELD_DIVEMODE:
+				*((dc_divemode_t *) value) = DC_DIVEMODE_FREEDIVE;
+				break;
 			default:
 				return DC_STATUS_UNSUPPORTED;
 			}
@@ -294,6 +352,20 @@ mares_nemo_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t c
 	unsigned int size = abstract->size;
 
 	if (parser->mode != parser->freedive) {
+		// Initial tank pressure.
+		unsigned int pressure = 0;
+		if (parser->extra == 12) {
+			const unsigned char *p = data + 2 + parser->sample_count * parser->sample_size;
+			pressure = array_uint16_le(p + parser->header + 4);
+		}
+
+		// Initial gas mix.
+		unsigned int gasmix_previous = 0xFFFFFFFF;
+		unsigned int gasmix = gasmix_previous;
+		if (parser->mode == AIR || parser->mode == NITROX) {
+			gasmix = 0;
+		}
+
 		unsigned int time = 0;
 		for (unsigned int i = 0; i < parser->sample_count; ++i) {
 			dc_sample_value_t sample = {0};
@@ -314,6 +386,13 @@ mares_nemo_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t c
 			sample.depth = depth / 10.0;
 			if (callback) callback (DC_SAMPLE_DEPTH, sample, userdata);
 
+			// Gas change.
+			if (gasmix != gasmix_previous) {
+				sample.gasmix = gasmix;
+				if (callback) callback (DC_SAMPLE_GASMIX, sample, userdata);
+				gasmix_previous = gasmix;
+			}
+
 			// Ascent rate
 			if (ascent) {
 				sample.event.type = SAMPLE_EVENT_ASCENT;
@@ -334,18 +413,27 @@ mares_nemo_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t c
 
 			// Deco stop
 			if (deco) {
-				sample.event.type = SAMPLE_EVENT_DECOSTOP;
-				sample.event.time = 0;
-				sample.event.flags = 0;
-				sample.event.value = 0;
-				if (callback) callback (DC_SAMPLE_EVENT, sample, userdata);
+				sample.deco.type = DC_DECO_DECOSTOP;
+			} else {
+				sample.deco.type = DC_DECO_NDL;
 			}
+			sample.deco.time = 0;
+			sample.deco.depth = 0.0;
+			if (callback) callback (DC_SAMPLE_DECO, sample, userdata);
 
 			// Pressure (1 bar).
 			if (parser->sample_size == 3) {
 				sample.pressure.tank = 0;
 				sample.pressure.value = data[idx + 2];
 				if (callback) callback (DC_SAMPLE_PRESSURE, sample, userdata);
+			} else if (parser->sample_size == 5) {
+				unsigned int type = (time / 20) % 3;
+				if (type == 0) {
+					pressure -= data[idx + 2] * 100;
+					sample.pressure.tank = 0;
+					sample.pressure.value = pressure / 100.0;
+					if (callback) callback (DC_SAMPLE_PRESSURE, sample, userdata);
+				}
 			}
 		}
 	} else {
