@@ -1,18 +1,18 @@
-/* 
+/*
  * libdivecomputer
- * 
+ *
  * Copyright (C) 2008 Jef Driesen
- * 
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
@@ -22,21 +22,21 @@
 #ifndef OCEANIC_VEO250_H
 #define OCEANIC_VEO250_H
 
-#include "device.h"
-#include "parser.h"
+#include <libdivecomputer/context.h>
+#include <libdivecomputer/device.h>
+#include <libdivecomputer/parser.h>
+
+#include <libdivecomputer/oceanic_veo250.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
-device_status_t
-oceanic_veo250_device_open (device_t **device, const char* name);
+dc_status_t
+oceanic_veo250_device_open (dc_device_t **device, dc_context_t *context, const char *name);
 
-device_status_t
-oceanic_veo250_device_keepalive (device_t *device);
-
-parser_status_t
-oceanic_veo250_parser_create (parser_t **parser, unsigned int model);
+dc_status_t
+oceanic_veo250_parser_create (dc_parser_t **parser, dc_context_t *context, unsigned int model);
 
 #ifdef __cplusplus
 }
