@@ -21,9 +21,9 @@
 
 #include <stdlib.h>
 
-#include <libdivecomputer/oceanic_veo250.h>
 #include <libdivecomputer/units.h>
 
+#include "oceanic_veo250.h"
 #include "oceanic_common.h"
 #include "context-private.h"
 #include "parser-private.h"
@@ -34,6 +34,7 @@
 #define REACTPRO 0x4247
 #define VEO200   0x424B
 #define VEO250   0x424C
+#define INSIGHT  0x425A
 #define REACTPROWHITE 0x4354
 
 typedef struct oceanic_veo250_parser_t oceanic_veo250_parser_t;
@@ -51,34 +52,33 @@ static dc_status_t oceanic_veo250_parser_set_data (dc_parser_t *abstract, const
 static dc_status_t oceanic_veo250_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime);
 static dc_status_t oceanic_veo250_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value);
 static dc_status_t oceanic_veo250_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
-static dc_status_t oceanic_veo250_parser_destroy (dc_parser_t *abstract);
 
 static const dc_parser_vtable_t oceanic_veo250_parser_vtable = {
+	sizeof(oceanic_veo250_parser_t),
 	DC_FAMILY_OCEANIC_VEO250,
 	oceanic_veo250_parser_set_data, /* set_data */
 	oceanic_veo250_parser_get_datetime, /* datetime */
 	oceanic_veo250_parser_get_field, /* fields */
 	oceanic_veo250_parser_samples_foreach, /* samples_foreach */
-	oceanic_veo250_parser_destroy /* destroy */
+	NULL /* destroy */
 };
 
 
 dc_status_t
 oceanic_veo250_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int model)
 {
+	oceanic_veo250_parser_t *parser = NULL;
+
 	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
-	oceanic_veo250_parser_t *parser = (oceanic_veo250_parser_t *) malloc (sizeof (oceanic_veo250_parser_t));
+	parser = (oceanic_veo250_parser_t *) dc_parser_allocate (context, &oceanic_veo250_parser_vtable);
 	if (parser == NULL) {
 		ERROR (context, "Failed to allocate memory.");
 		return DC_STATUS_NOMEMORY;
 	}
 
-	// Initialize the base class.
-	parser_init (&parser->base, context, &oceanic_veo250_parser_vtable);
-
 	// Set the default values.
 	parser->model = model;
 	parser->cached = 0;
@@ -91,16 +91,6 @@ oceanic_veo250_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 }
 
 
-static dc_status_t
-oceanic_veo250_parser_destroy (dc_parser_t *abstract)
-{
-	// Free memory.
-	free (abstract);
-
-	return DC_STATUS_SUCCESS;
-}
-
-
 static dc_status_t
 oceanic_veo250_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size)
 {
@@ -132,6 +122,7 @@ oceanic_veo250_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *dateti
 		datetime->hour   = p[3];
 		datetime->minute = p[2];
 		datetime->second = 0;
+		datetime->timezone = DC_TIMEZONE_NONE;
 
 		if (parser->model == VEO200 || parser->model == VEO250)
 			datetime->year += 3;
@@ -173,7 +164,7 @@ oceanic_veo250_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, un
 	if (value) {
 		switch (type) {
 		case DC_FIELD_DIVETIME:
-			*((unsigned int *) value) = data[footer + 3] * 60;
+			*((unsigned int *) value) = data[footer + 3] * 60 + data[footer + 4] * 3600;
 			break;
 		case DC_FIELD_MAXDEPTH:
 			*((double *) value) = parser->maxdepth;
@@ -258,7 +249,8 @@ oceanic_veo250_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback
 
 		// Temperature (°F)
 		unsigned int temperature;
-		if (parser->model == REACTPRO || parser->model == REACTPROWHITE) {
+		if (parser->model == REACTPRO || parser->model == REACTPROWHITE ||
+			parser->model == INSIGHT) {
 			temperature = data[offset + 6];
 		} else {
 			temperature = data[offset + 7];
@@ -266,6 +258,31 @@ oceanic_veo250_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback
 		sample.temperature = (temperature - 32.0) * (5.0 / 9.0);
 		if (callback) callback (DC_SAMPLE_TEMPERATURE, sample, userdata);
 
+		// NDL / Deco
+		unsigned int have_deco = 0;
+		unsigned int decostop = 0, decotime = 0;
+		if (parser->model == REACTPRO || parser->model == REACTPROWHITE ||
+			parser->model == INSIGHT) {
+			decostop = (data[offset + 7] & 0xF0) >> 4;
+			decotime = ((data[offset + 3] & 0xC0) << 2) | data[offset + 4];
+			have_deco = 1;
+		} else {
+			decostop = (data[offset + 5] & 0xF0) >> 4;
+			decotime = array_uint16_le(data + offset + 4) & 0x0FFF;
+			have_deco = 1;
+		}
+		if (have_deco) {
+			if (decostop) {
+				sample.deco.type = DC_DECO_DECOSTOP;
+				sample.deco.depth = decostop * 10 * FEET;
+			} else {
+				sample.deco.type = DC_DECO_NDL;
+				sample.deco.depth = 0.0;
+			}
+			sample.deco.time = decotime * 60;
+			if (callback) callback (DC_SAMPLE_DECO, sample, userdata);
+		}
+
 		offset += PAGESIZE / 2;
 	}
 
