@@ -1,18 +1,18 @@
-/* 
+/*
  * libdivecomputer
- * 
+ *
  * Copyright (C) 2008 Jef Driesen
- * 
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
@@ -22,9 +22,9 @@
 #include <stdlib.h>
 #include <string.h>	// memcmp
 
-#include <libdivecomputer/reefnet_sensusultra.h>
 #include <libdivecomputer/units.h>
 
+#include "reefnet_sensusultra.h"
 #include "context-private.h"
 #include "parser-private.h"
 #include "array.h"
@@ -51,34 +51,33 @@ static dc_status_t reefnet_sensusultra_parser_set_data (dc_parser_t *abstract, c
 static dc_status_t reefnet_sensusultra_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime);
 static dc_status_t reefnet_sensusultra_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value);
 static dc_status_t reefnet_sensusultra_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
-static dc_status_t reefnet_sensusultra_parser_destroy (dc_parser_t *abstract);
 
 static const dc_parser_vtable_t reefnet_sensusultra_parser_vtable = {
+	sizeof(reefnet_sensusultra_parser_t),
 	DC_FAMILY_REEFNET_SENSUSULTRA,
 	reefnet_sensusultra_parser_set_data, /* set_data */
 	reefnet_sensusultra_parser_get_datetime, /* datetime */
 	reefnet_sensusultra_parser_get_field, /* fields */
 	reefnet_sensusultra_parser_samples_foreach, /* samples_foreach */
-	reefnet_sensusultra_parser_destroy /* destroy */
+	NULL /* destroy */
 };
 
 
 dc_status_t
 reefnet_sensusultra_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int devtime, dc_ticks_t systime)
 {
+	reefnet_sensusultra_parser_t *parser = NULL;
+
 	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
-	reefnet_sensusultra_parser_t *parser = (reefnet_sensusultra_parser_t *) malloc (sizeof (reefnet_sensusultra_parser_t));
+	parser = (reefnet_sensusultra_parser_t *) dc_parser_allocate (context, &reefnet_sensusultra_parser_vtable);
 	if (parser == NULL) {
 		ERROR (context, "Failed to allocate memory.");
 		return DC_STATUS_NOMEMORY;
 	}
 
-	// Initialize the base class.
-	parser_init (&parser->base, context, &reefnet_sensusultra_parser_vtable);
-
 	// Set the default values.
 	parser->atmospheric = ATM;
 	parser->hydrostatic = 1025.0 * GRAVITY;
@@ -94,16 +93,6 @@ reefnet_sensusultra_parser_create (dc_parser_t **out, dc_context_t *context, uns
 }
 
 
-static dc_status_t
-reefnet_sensusultra_parser_destroy (dc_parser_t *abstract)
-{
-	// Free memory.	
-	free (abstract);
-
-	return DC_STATUS_SUCCESS;
-}
-
-
 static dc_status_t
 reefnet_sensusultra_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size)
 {
@@ -201,6 +190,9 @@ reefnet_sensusultra_parser_get_field (dc_parser_t *abstract, dc_field_type_t typ
 		case DC_FIELD_GASMIX_COUNT:
 			*((unsigned int *) value) = 0;
 			break;
+		case DC_FIELD_DIVEMODE:
+			*((dc_divemode_t *) value) = DC_DIVEMODE_GAUGE;
+			break;
 		default:
 			return DC_STATUS_UNSUPPORTED;
 		}
@@ -231,8 +223,8 @@ reefnet_sensusultra_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 			unsigned int interval = array_uint16_le (data + offset + 8);
 
 			offset += 16;
-			while (offset + sizeof (footer) <= size && 
-				memcmp (data + offset, footer, sizeof (footer)) != 0) 
+			while (offset + sizeof (footer) <= size &&
+				memcmp (data + offset, footer, sizeof (footer)) != 0)
 			{
 				dc_sample_value_t sample = {0};
 
