@@ -35,38 +35,41 @@
 #define ESC_END   0xDC
 #define ESC_ESC   0xDD
 
-#define EXITCODE(n) ((n) < 0 ? (n) : 0)
-
 dc_status_t
 shearwater_common_open (shearwater_common_device_t *device, dc_context_t *context, const char *name)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
+
 	// Open the device.
-	int rc = serial_open (&device->port, context, name);
-	if (rc == -1) {
+	status = dc_serial_open (&device->port, context, name);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to open the serial port.");
-		return DC_STATUS_IO;
+		return status;
 	}
 
 	// Set the serial communication protocol (115200 8N1).
-	rc = serial_configure (device->port, 115200, 8, SERIAL_PARITY_NONE, 1, SERIAL_FLOWCONTROL_NONE);
-	if (rc == -1) {
+	status = dc_serial_configure (device->port, 115200, 8, DC_PARITY_NONE, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the terminal attributes.");
-		serial_close (device->port);
-		return DC_STATUS_IO;
+		goto error_close;
 	}
 
 	// Set the timeout for receiving data (3000ms).
-	if (serial_set_timeout (device->port, 3000) == -1) {
+	status = dc_serial_set_timeout (device->port, 3000);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
-		serial_close (device->port);
-		return DC_STATUS_IO;
+		goto error_close;
 	}
 
 	// Make sure everything is in a sane state.
-	serial_sleep (device->port, 300);
-	serial_flush (device->port, SERIAL_QUEUE_BOTH);
+	dc_serial_sleep (device->port, 300);
+	dc_serial_purge (device->port, DC_DIRECTION_ALL);
 
 	return DC_STATUS_SUCCESS;
+
+error_close:
+	dc_serial_close (device->port);
+	return status;
 }
 
 
@@ -74,11 +77,7 @@ dc_status_t
 shearwater_common_close (shearwater_common_device_t *device)
 {
 	// Close the device.
-	if (serial_close (device->port) == -1) {
-		return DC_STATUS_IO;
-	}
-
-	return DC_STATUS_SUCCESS;
+	return dc_serial_close (device->port);
 }
 
 
@@ -141,10 +140,10 @@ shearwater_common_decompress_xor (unsigned char *data, unsigned int size)
 }
 
 
-static int
+static dc_status_t
 shearwater_common_slip_write (shearwater_common_device_t *device, const unsigned char data[], unsigned int size)
 {
-	int n = 0;
+	dc_status_t status = DC_STATUS_SUCCESS;
 	const unsigned char end[] = {END};
 	const unsigned char esc_end[] = {ESC, ESC_END};
 	const unsigned char esc_esc[] = {ESC, ESC_ESC};
@@ -154,9 +153,9 @@ shearwater_common_slip_write (shearwater_common_device_t *device, const unsigned
 #if 0
 	// Send an initial END character to flush out any data that may have
 	// accumulated in the receiver due to line noise.
-	n = serial_write (device->port, end, sizeof (end));
-	if (n != sizeof (end)) {
-		return EXITCODE(n);
+	status = dc_serial_write (device->port, end, sizeof (end), NULL);
+	if (status != DC_STATUS_SUCCESS) {
+		return status;
 	}
 #endif
 
@@ -183,9 +182,9 @@ shearwater_common_slip_write (shearwater_common_device_t *device, const unsigned
 
 		// Flush the buffer if necessary.
 		if (nbytes + len + sizeof(end) > sizeof(buffer)) {
-			n = serial_write (device->port, buffer, nbytes);
-			if (n != nbytes) {
-				return EXITCODE(n);
+			status = dc_serial_write (device->port, buffer, nbytes, NULL);
+			if (status != DC_STATUS_SUCCESS) {
+				return status;
 			}
 
 			nbytes = 0;
@@ -201,18 +200,19 @@ shearwater_common_slip_write (shearwater_common_device_t *device, const unsigned
 	nbytes += sizeof(end);
 
 	// Flush the buffer.
-	n = serial_write (device->port, buffer, nbytes);
-	if (n != nbytes) {
-		return EXITCODE(n);
+	status = dc_serial_write (device->port, buffer, nbytes, NULL);
+	if (status != DC_STATUS_SUCCESS) {
+		return status;
 	}
 
-	return size;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static int
-shearwater_common_slip_read (shearwater_common_device_t *device, unsigned char data[], unsigned int size)
+static dc_status_t
+shearwater_common_slip_read (shearwater_common_device_t *device, unsigned char data[], unsigned int size, unsigned int *actual)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
 	unsigned int received = 0;
 
 	// Read bytes until a complete packet has been received. If the
@@ -221,12 +221,11 @@ shearwater_common_slip_read (shearwater_common_device_t *device, unsigned char d
 	// than the supplied buffer size.
 	while (1) {
 		unsigned char c = 0;
-		int n = 0;
 
 		// Get a single character to process.
-		n = serial_read (device->port, &c, 1);
-		if (n != 1) {
-			return EXITCODE(n);
+		status = dc_serial_read (device->port, &c, 1, NULL);
+		if (status != DC_STATUS_SUCCESS) {
+			return status;
 		}
 
 		switch (c) {
@@ -237,15 +236,15 @@ shearwater_common_slip_read (shearwater_common_device_t *device, unsigned char d
 			// packets generated by the duplicate END characters which
 			// are sent to try to detect line noise.
 			if (received)
-				return received;
+				goto done;
 			else
 				break;
 		case ESC:
 			// If it's an ESC character, get another character and then
 			// figure out what to store in the packet based on that.
-			n = serial_read (device->port, &c, 1);
-			if (n != 1) {
-				return EXITCODE(n);
+			status = dc_serial_read (device->port, &c, 1, NULL);
+			if (status != DC_STATUS_SUCCESS) {
+				return status;
 			}
 
 			// If it's not one of the two escaped characters, then we
@@ -267,16 +266,25 @@ shearwater_common_slip_read (shearwater_common_device_t *device, unsigned char d
 		}
 	}
 
-	return received;
+done:
+
+	if (received > size)
+		return DC_STATUS_PROTOCOL;
+
+	if (actual)
+		*actual = received;
+
+	return DC_STATUS_SUCCESS;
 }
 
 
 dc_status_t
 shearwater_common_transfer (shearwater_common_device_t *device, const unsigned char input[], unsigned int isize, unsigned char output[], unsigned int osize, unsigned int *actual)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
 	dc_device_t *abstract = (dc_device_t *) device;
 	unsigned char packet[SZ_PACKET + 4];
-	int n = 0;
+	unsigned int n = 0;
 
 	if (isize > SZ_PACKET || osize > SZ_PACKET)
 		return DC_STATUS_INVALIDARGS;
@@ -292,13 +300,10 @@ shearwater_common_transfer (shearwater_common_device_t *device, const unsigned c
 	memcpy (packet + 4, input, isize);
 
 	// Send the request packet.
-	n = shearwater_common_slip_write (device, packet, isize + 4);
-	if (n != isize + 4) {
+	status = shearwater_common_slip_write (device, packet, isize + 4);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the request packet.");
-		if (n < 0)
-			return DC_STATUS_IO;
-		else
-			return DC_STATUS_TIMEOUT;
+		return status;
 	}
 
 	// Return early if no response packet is requested.
@@ -309,15 +314,10 @@ shearwater_common_transfer (shearwater_common_device_t *device, const unsigned c
 	}
 
 	// Receive the response packet.
-	n = shearwater_common_slip_read (device, packet, sizeof (packet));
-	if (n <= 0 || n > sizeof (packet)) {
+	status = shearwater_common_slip_read (device, packet, sizeof (packet), &n);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the response packet.");
-		if (n < 0)
-			return DC_STATUS_IO;
-		else if (n > sizeof (packet))
-			return DC_STATUS_PROTOCOL;
-		else
-			return DC_STATUS_TIMEOUT;
+		return status;
 	}
 
 	// Validate the packet header.
@@ -342,7 +342,7 @@ shearwater_common_transfer (shearwater_common_device_t *device, const unsigned c
 
 
 dc_status_t
-shearwater_common_download (shearwater_common_device_t *device, dc_buffer_t *buffer, unsigned int address, unsigned int size, unsigned int compression)
+shearwater_common_download (shearwater_common_device_t *device, dc_buffer_t *buffer, unsigned int address, unsigned int size, unsigned int compression, dc_event_progress_t *progress)
 {
 	dc_device_t *abstract = (dc_device_t *) device;
 	dc_status_t rc = DC_STATUS_SUCCESS;
@@ -370,9 +370,11 @@ shearwater_common_download (shearwater_common_device_t *device, dc_buffer_t *buf
 	}
 
 	// Enable progress notifications.
-	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
-	progress.maximum = 3 + size + 1;
-	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
+	unsigned int initial = 0, current = 0, maximum = 3 + size + 1;
+	if (progress) {
+		initial = progress->current;
+		device_event_emit (abstract, DC_EVENT_PROGRESS, progress);
+	}
 
 	// Transfer the init request.
 	rc = shearwater_common_transfer (device, req_init, sizeof (req_init), response, 3, &n);
@@ -387,8 +389,11 @@ shearwater_common_download (shearwater_common_device_t *device, dc_buffer_t *buf
 	}
 
 	// Update and emit a progress event.
-	progress.current += 3;
-	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
+	if (progress) {
+		current += 3;
+		progress->current = initial + STEP (current, maximum);
+		device_event_emit (abstract, DC_EVENT_PROGRESS, progress);
+	}
 
 	unsigned int done = 0;
 	unsigned char block = 1;
@@ -415,8 +420,11 @@ shearwater_common_download (shearwater_common_device_t *device, dc_buffer_t *buf
 		}
 
 		// Update and emit a progress event.
-		progress.current += length;
-		device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
+		if (progress) {
+			current += length;
+			progress->current = initial + STEP (current, maximum);
+			device_event_emit (abstract, DC_EVENT_PROGRESS, progress);
+		}
 
 		if (compression) {
 			if (shearwater_common_decompress_lre (response + 2, length, buffer, &done) != 0) {
@@ -454,8 +462,11 @@ shearwater_common_download (shearwater_common_device_t *device, dc_buffer_t *buf
 	}
 
 	// Update and emit a progress event.
-	progress.current += 1;
-	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
+	if (progress) {
+		current += 1;
+		progress->current = initial + STEP (current, maximum);
+		device_event_emit (abstract, DC_EVENT_PROGRESS, progress);
+	}
 
 	return DC_STATUS_SUCCESS;
 }
