@@ -22,19 +22,14 @@
 #include <string.h> // memcmp, memcpy
 #include <stdlib.h> // malloc, free
 
-#include <libdivecomputer/shearwater_predator.h>
-
+#include "shearwater_predator.h"
 #include "shearwater_common.h"
-
 #include "context-private.h"
 #include "device-private.h"
 #include "array.h"
 
 #define ISINSTANCE(device) dc_device_isinstance((device), &shearwater_predator_device_vtable)
 
-#define PREDATOR 2
-#define PETREL   3
-
 #define SZ_BLOCK   0x80
 #define SZ_MEMORY  0x20080
 
@@ -52,63 +47,61 @@ static dc_status_t shearwater_predator_device_foreach (dc_device_t *abstract, dc
 static dc_status_t shearwater_predator_device_close (dc_device_t *abstract);
 
 static const dc_device_vtable_t shearwater_predator_device_vtable = {
+	sizeof(shearwater_predator_device_t),
 	DC_FAMILY_SHEARWATER_PREDATOR,
 	shearwater_predator_device_set_fingerprint, /* set_fingerprint */
 	NULL, /* read */
 	NULL, /* write */
 	shearwater_predator_device_dump, /* dump */
 	shearwater_predator_device_foreach, /* foreach */
+	NULL, /* timesync */
 	shearwater_predator_device_close /* close */
 };
 
+static dc_status_t
+shearwater_predator_extract_dives (dc_device_t *device, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata);
 
 dc_status_t
 shearwater_predator_device_open (dc_device_t **out, dc_context_t *context, const char *name)
 {
-	dc_status_t rc = DC_STATUS_SUCCESS;
+	dc_status_t status = DC_STATUS_SUCCESS;
+	shearwater_predator_device_t *device = NULL;
 
 	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
-	shearwater_predator_device_t *device = (shearwater_predator_device_t *) malloc (sizeof (shearwater_predator_device_t));
+	device = (shearwater_predator_device_t *) dc_device_allocate (context, &shearwater_predator_device_vtable);
 	if (device == NULL) {
 		ERROR (context, "Failed to allocate memory.");
 		return DC_STATUS_NOMEMORY;
 	}
 
-	// Initialize the base class.
-	device_init (&device->base.base, context, &shearwater_predator_device_vtable);
-
 	// Set the default values.
 	memset (device->fingerprint, 0, sizeof (device->fingerprint));
 
 	// Open the device.
-	rc = shearwater_common_open (&device->base, context, name);
-	if (rc != DC_STATUS_SUCCESS) {
-		free (device);
-		return rc;
+	status = shearwater_common_open (&device->base, context, name);
+	if (status != DC_STATUS_SUCCESS) {
+		goto error_free;
 	}
 
 	*out = (dc_device_t *) device;
 
 	return DC_STATUS_SUCCESS;
+
+error_free:
+	dc_device_deallocate ((dc_device_t *) device);
+	return status;
 }
 
 
 static dc_status_t
 shearwater_predator_device_close (dc_device_t *abstract)
 {
-	dc_status_t rc = DC_STATUS_SUCCESS;
 	shearwater_common_device_t *device = (shearwater_common_device_t *) abstract;
 
-	// Close the device.
-	rc = shearwater_common_close (device);
-
-	// Free memory.
-	free (device);
-
-	return rc;
+	return shearwater_common_close (device);
 }
 
 
@@ -140,7 +133,12 @@ shearwater_predator_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 		return DC_STATUS_NOMEMORY;
 	}
 
-	return shearwater_common_download (device, buffer, 0xDD000000, SZ_MEMORY, 0);
+	// Enable progress notifications.
+	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
+	progress.current = 0;
+	progress.maximum = NSTEPS;
+
+	return shearwater_common_download (device, buffer, 0xDD000000, SZ_MEMORY, 0, &progress);
 }
 
 
@@ -161,8 +159,8 @@ shearwater_predator_device_foreach (dc_device_t *abstract, dc_dive_callback_t ca
 	unsigned char *data = dc_buffer_get_data (buffer);
 	dc_event_devinfo_t devinfo;
 	devinfo.model = data[0x2000D];
-	devinfo.firmware = data[0x2000A];
-	devinfo.serial = array_uint32_le (data + 0x20002);
+	devinfo.firmware = bcd2dec (data[0x2000A]);
+	devinfo.serial = array_uint32_be (data + 0x20002);
 	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
 
 	rc = shearwater_predator_extract_dives (abstract, data, SZ_MEMORY, callback, userdata);
@@ -344,7 +342,7 @@ shearwater_predator_extract_petrel (dc_device_t *abstract, const unsigned char d
 }
 
 
-dc_status_t
+static dc_status_t
 shearwater_predator_extract_dives (dc_device_t *abstract, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata)
 {
 	if (abstract && !ISINSTANCE (abstract))
