@@ -1,18 +1,18 @@
-/* 
+/*
  * libdivecomputer
- * 
+ *
  * Copyright (C) 2008 Jef Driesen
- * 
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
@@ -22,8 +22,7 @@
 #include <stdlib.h>
 #include <string.h>	// memcmp
 
-#include <libdivecomputer/suunto_d9.h>
-
+#include "suunto_d9.h"
 #include "context-private.h"
 #include "parser-private.h"
 #include "array.h"
@@ -31,6 +30,7 @@
 #define ISINSTANCE(parser) dc_parser_isinstance((parser), &suunto_d9_parser_vtable)
 
 #define MAXPARAMS 3
+#define NGASMIXES 11
 
 #define D9       0x0E
 #define D6       0x0F
@@ -44,12 +44,24 @@
 #define D6i      0x1A
 #define D9tx     0x1B
 #define DX       0x1C
+#define VYPERNOVO 0x1D
+#define ZOOPNOVO  0x1E
+#define D4F       0x20
+
+#define ID_D6I_V1_MIX2 0x1871C062
+#define ID_D6I_V1_MIX3 0x1871C063
+#define ID_D6I_V2      0x18724062
+#define ID_D4I_V1      ID_D6I_V1_MIX2
+#define ID_D4I_V2      ID_D6I_V2
+#define ID_DX_V1       0x18922062
+#define ID_DX_V2       0x18924062
 
 #define AIR      0
 #define NITROX   1
 #define GAUGE    2
 #define FREEDIVE 3
 #define MIXED    4
+#define CCR      5
 
 #define SAFETYSTOP   (1 << 0)
 #define DECOSTOP     (1 << 1)
@@ -60,6 +72,14 @@ typedef struct suunto_d9_parser_t suunto_d9_parser_t;
 struct suunto_d9_parser_t {
 	dc_parser_t base;
 	unsigned int model;
+	// Cached fields.
+	unsigned int cached;
+	unsigned int mode;
+	unsigned int ngasmixes;
+	unsigned int oxygen[NGASMIXES];
+	unsigned int helium[NGASMIXES];
+	unsigned int gasmix;
+	unsigned int config;
 };
 
 typedef struct sample_info_t {
@@ -73,36 +93,169 @@ static dc_status_t suunto_d9_parser_set_data (dc_parser_t *abstract, const unsig
 static dc_status_t suunto_d9_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime);
 static dc_status_t suunto_d9_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value);
 static dc_status_t suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
-static dc_status_t suunto_d9_parser_destroy (dc_parser_t *abstract);
 
 static const dc_parser_vtable_t suunto_d9_parser_vtable = {
+	sizeof(suunto_d9_parser_t),
 	DC_FAMILY_SUUNTO_D9,
 	suunto_d9_parser_set_data, /* set_data */
 	suunto_d9_parser_get_datetime, /* datetime */
 	suunto_d9_parser_get_field, /* fields */
 	suunto_d9_parser_samples_foreach, /* samples_foreach */
-	suunto_d9_parser_destroy /* destroy */
+	NULL /* destroy */
 };
 
+static unsigned int
+suunto_d9_parser_find_gasmix (suunto_d9_parser_t *parser, unsigned int o2, unsigned int he)
+{
+	// Find the gasmix in the list.
+	unsigned int i = 0;
+	while (i < parser->ngasmixes) {
+		if (o2 == parser->oxygen[i] && he == parser->helium[i])
+			break;
+		i++;
+	}
+
+	return i;
+}
+
+static dc_status_t
+suunto_d9_parser_cache (suunto_d9_parser_t *parser)
+{
+	const unsigned char *data = parser->base.data;
+	unsigned int size = parser->base.size;
+
+	if (parser->cached) {
+		return DC_STATUS_SUCCESS;
+	}
+
+	// Get the logbook id tag.
+	unsigned int id = array_uint32_le (data + 1);
+
+	// Gasmix information.
+	unsigned int gasmode_offset = 0x19;
+	unsigned int gasmix_offset = 0x21;
+	unsigned int gasmix_count = 3;
+	if (parser->model == HELO2) {
+		gasmode_offset = 0x1F;
+		gasmix_offset = 0x54;
+		gasmix_count = 8;
+	} else if (parser->model == D4i || parser->model == ZOOPNOVO ||
+		parser->model == D4F) {
+		gasmode_offset = 0x1D;
+		if (id == ID_D4I_V2)
+			gasmix_offset = 0x67;
+		else
+			gasmix_offset = 0x5F;
+		gasmix_count = 1;
+	} else if (parser->model == D6i || parser->model == VYPERNOVO) {
+		gasmode_offset = 0x1D;
+		if (id == ID_D6I_V2)
+			gasmix_offset = 0x67;
+		else
+			gasmix_offset = 0x5F;
+		if (id == ID_D6I_V1_MIX3 || id == ID_D6I_V2)
+			gasmix_count = 3;
+		else
+			gasmix_count = 2;
+	} else if (parser->model == D9tx) {
+		gasmode_offset = 0x1D;
+		gasmix_offset = 0x87;
+		gasmix_count = 8;
+	} else if (parser->model == DX) {
+		gasmode_offset = 0x21;
+		if (id == ID_DX_V2)
+			gasmix_offset = 0xC3;
+		else
+			gasmix_offset = 0xC1;
+		gasmix_count = 11;
+	}
+
+	// Offset to the configuration data.
+	unsigned int config = 0x3A;
+	if (parser->model == D4) {
+		config += 1;
+	} else if (parser->model == HELO2 || parser->model == D4i ||
+		parser->model == D6i || parser->model == D9tx ||
+		parser->model == DX || parser->model == ZOOPNOVO ||
+		parser->model == VYPERNOVO || parser->model == D4F) {
+		config = gasmix_offset + gasmix_count * 6;
+	}
+	if (config + 1 > size)
+		return DC_STATUS_DATAFORMAT;
+
+	// Cache the data for later use.
+	parser->mode = data[gasmode_offset];
+	parser->gasmix = 0;
+	if (parser->mode == GAUGE || parser->mode == FREEDIVE) {
+		parser->ngasmixes = 0;
+	} else if (parser->mode == AIR) {
+		parser->oxygen[0] = 21;
+		parser->helium[0] = 0;
+		parser->ngasmixes = 1;
+	} else {
+		parser->ngasmixes = 0;
+		for (unsigned int i = 0; i < gasmix_count; ++i) {
+			if (parser->model == HELO2 || parser->model == D4i ||
+				parser->model == D6i || parser->model == D9tx ||
+				parser->model == DX || parser->model == ZOOPNOVO ||
+				parser->model == VYPERNOVO || parser->model == D4F) {
+				parser->oxygen[i] = data[gasmix_offset + 6 * i + 1];
+				parser->helium[i] = data[gasmix_offset + 6 * i + 2];
+			} else {
+				unsigned int oxygen = data[gasmix_offset + i];
+				if (oxygen == 0x00 || oxygen == 0xFF)
+					break;
+				parser->oxygen[i] = oxygen;
+				parser->helium[i] = 0;
+			}
+			parser->ngasmixes++;
+		}
+
+		// Initial gasmix.
+		if (parser->model == HELO2) {
+			parser->gasmix = data[0x26];
+		} else if (parser->model == D4i || parser->model == D6i ||
+			parser->model == D9tx || parser->model == ZOOPNOVO ||
+			parser->model == VYPERNOVO || parser->model == D4F) {
+			if (id == ID_D4I_V2 || id == ID_D6I_V2) {
+				parser->gasmix = data[0x2D];
+			} else {
+				parser->gasmix = data[0x28];
+			}
+		}
+	}
+	parser->config = config;
+	parser->cached = 1;
+
+	return DC_STATUS_SUCCESS;
+}
 
 dc_status_t
 suunto_d9_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int model)
 {
+	suunto_d9_parser_t *parser = NULL;
+
 	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
-	suunto_d9_parser_t *parser = (suunto_d9_parser_t *) malloc (sizeof (suunto_d9_parser_t));
+	parser = (suunto_d9_parser_t *) dc_parser_allocate (context, &suunto_d9_parser_vtable);
 	if (parser == NULL) {
 		ERROR (context, "Failed to allocate memory.");
 		return DC_STATUS_NOMEMORY;
 	}
 
-	// Initialize the base class.
-	parser_init (&parser->base, context, &suunto_d9_parser_vtable);
-
 	// Set the default values.
 	parser->model = model;
+	parser->cached = 0;
+	parser->mode = AIR;
+	parser->ngasmixes = 0;
+	for (unsigned int i = 0; i < NGASMIXES; ++i) {
+		parser->oxygen[i] = 0;
+		parser->helium[i] = 0;
+	}
+	parser->gasmix = 0;
+	parser->config = 0;
 
 	*out = (dc_parser_t*) parser;
 
@@ -110,19 +263,22 @@ suunto_d9_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int
 }
 
 
-static dc_status_t
-suunto_d9_parser_destroy (dc_parser_t *abstract)
-{
-	// Free memory.	
-	free (abstract);
-
-	return DC_STATUS_SUCCESS;
-}
-
-
 static dc_status_t
 suunto_d9_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size)
 {
+	suunto_d9_parser_t *parser = (suunto_d9_parser_t *) abstract;
+
+	// Reset the cache.
+	parser->cached = 0;
+	parser->mode = AIR;
+	parser->ngasmixes = 0;
+	for (unsigned int i = 0; i < NGASMIXES; ++i) {
+		parser->oxygen[i] = 0;
+		parser->helium[i] = 0;
+	}
+	parser->gasmix = 0;
+	parser->config = 0;
+
 	return DC_STATUS_SUCCESS;
 }
 
@@ -135,7 +291,9 @@ suunto_d9_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime)
 	unsigned int offset = 0x11;
 	if (parser->model == HELO2 || parser->model == DX)
 		offset = 0x17;
-	else if (parser->model == D4i || parser->model == D6i || parser->model == D9tx)
+	else if (parser->model == D4i || parser->model == D6i ||
+		parser->model == D9tx || parser->model == ZOOPNOVO ||
+		parser->model == VYPERNOVO || parser->model == D4F)
 		offset = 0x13;
 
 	if (abstract->size < offset + 7)
@@ -145,7 +303,9 @@ suunto_d9_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime)
 
 	if (datetime) {
 		if (parser->model == D4i || parser->model == D6i ||
-			parser->model == D9tx || parser->model == DX) {
+			parser->model == D9tx || parser->model == DX ||
+			parser->model == ZOOPNOVO || parser->model == VYPERNOVO ||
+			parser->model == D4F) {
 			datetime->year   = p[0] + (p[1] << 8);
 			datetime->month  = p[2];
 			datetime->day    = p[3];
@@ -160,6 +320,7 @@ suunto_d9_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime)
 			datetime->month  = p[5];
 			datetime->day    = p[6];
 		}
+		datetime->timezone = DC_TIMEZONE_NONE;
 	}
 
 	return DC_STATUS_SUCCESS;
@@ -174,49 +335,10 @@ suunto_d9_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigne
 	const unsigned char *data = abstract->data;
 	unsigned int size = abstract->size;
 
-	// Gasmix information.
-	unsigned int gasmix_offset = 0x21, gasmix_count = 3;
-	if (parser->model == HELO2) {
-		gasmix_offset = 0x54;
-		gasmix_count = 8;
-	} else if (parser->model == D4i) {
-		gasmix_offset = 0x5F;
-		gasmix_count = 1;
-	} else if (parser->model == D6i) {
-		gasmix_offset = 0x5F;
-		if (data[1] == 0x63)
-			gasmix_count = 3;
-		else
-			gasmix_count = 2;
-	} else if (parser->model == D9tx) {
-		gasmix_offset = 0x87;
-		gasmix_count = 8;
-	} else if (parser->model == DX) {
-		gasmix_offset = 0xC1;
-		gasmix_count = 11;
-	}
-
-	// Offset to the configuration data.
-	unsigned int config = 0x3A;
-	if (parser->model == D4) {
-		config += 1;
-	} else if (parser->model == HELO2 || parser->model == D4i ||
-		parser->model == D6i || parser->model == D9tx ||
-		parser->model == DX) {
-		config = gasmix_offset + gasmix_count * 6;
-	}
-	if (config + 1 > size)
-		return DC_STATUS_DATAFORMAT;
-
-	// Gas model
-	unsigned int gasmodel_offset = 0x19;
-	if (parser->model == HELO2)
-		gasmodel_offset = 0x1F;
-	else if (parser->model == D4i || parser->model == D6i || parser->model == D9tx)
-		gasmodel_offset = 0x1D;
-	else if (parser->model == DX)
-		gasmodel_offset = 0x21;
-	unsigned int gasmodel = data[gasmodel_offset];
+	// Cache the gas mix data.
+	dc_status_t rc = suunto_d9_parser_cache (parser);
+	if (rc != DC_STATUS_SUCCESS)
+		return rc;
 
 	dc_gasmix_t *gasmix = (dc_gasmix_t *) value;
 
@@ -226,7 +348,9 @@ suunto_d9_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigne
 			if (parser->model == D4)
 				*((unsigned int *) value) = array_uint16_le (data + 0x0B);
 			else if (parser->model == D4i || parser->model == D6i ||
-				parser->model == D9tx || parser->model == DX)
+				parser->model == D9tx || parser->model == DX ||
+				parser->model == ZOOPNOVO || parser->model == VYPERNOVO ||
+				parser->model == D4F)
 				*((unsigned int *) value) = array_uint16_le (data + 0x0D);
 			else if (parser->model == HELO2)
 				*((unsigned int *) value) = array_uint16_le (data + 0x0D) * 60;
@@ -237,27 +361,33 @@ suunto_d9_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigne
 			*((double *) value) = array_uint16_le (data + 0x09) / 100.0;
 			break;
 		case DC_FIELD_GASMIX_COUNT:
-			if (gasmodel == AIR) {
-				*((unsigned int *) value) = 1;
-			} else {
-				*((unsigned int *) value) = gasmix_count;
-			}
+			*((unsigned int *) value) = parser->ngasmixes;
 			break;
 		case DC_FIELD_GASMIX:
-			if (gasmodel == AIR) {
-				gasmix->helium = 0.0;
-				gasmix->oxygen = 0.21;
-			} else if (parser->model == HELO2 || parser->model == D4i ||
-				parser->model == D6i || parser->model == D9tx ||
-				parser->model == DX) {
-				gasmix->helium = data[gasmix_offset + 6 * flags + 2] / 100.0;
-				gasmix->oxygen = data[gasmix_offset + 6 * flags + 1] / 100.0;
-			} else {
-				gasmix->helium = 0.0;
-				gasmix->oxygen = data[gasmix_offset + flags] / 100.0;
-			}
+			gasmix->helium = parser->helium[flags] / 100.0;
+			gasmix->oxygen = parser->oxygen[flags] / 100.0;
 			gasmix->nitrogen = 1.0 - gasmix->oxygen - gasmix->helium;
 			break;
+		case DC_FIELD_DIVEMODE:
+			switch (parser->mode) {
+			case AIR:
+			case NITROX:
+			case MIXED:
+				*((dc_divemode_t *) value) = DC_DIVEMODE_OC;
+				break;
+			case GAUGE:
+				*((dc_divemode_t *) value) = DC_DIVEMODE_GAUGE;
+				break;
+			case FREEDIVE:
+				*((dc_divemode_t *) value) = DC_DIVEMODE_FREEDIVE;
+				break;
+			case CCR:
+				*((dc_divemode_t *) value) = DC_DIVEMODE_CC;
+				break;
+			default:
+				return DC_STATUS_DATAFORMAT;
+			}
+			break;
 		default:
 			return DC_STATUS_UNSUPPORTED;
 		}
@@ -275,44 +405,17 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 	const unsigned char *data = abstract->data;
 	unsigned int size = abstract->size;
 
-	// Gasmix information.
-	unsigned int gasmix_offset = 0x21, gasmix_count = 3;
-	if (parser->model == HELO2) {
-		gasmix_offset = 0x54;
-		gasmix_count = 8;
-	} else if (parser->model == D4i) {
-		gasmix_offset = 0x5F;
-		gasmix_count = 1;
-	} else if (parser->model == D6i) {
-		gasmix_offset = 0x5F;
-		if (data[1] == 0x63)
-			gasmix_count = 3;
-		else
-			gasmix_count = 2;
-	} else if (parser->model == D9tx) {
-		gasmix_offset = 0x87;
-		gasmix_count = 8;
-	} else if (parser->model == DX) {
-		gasmix_offset = 0xC1;
-		gasmix_count = 11;
-	}
-
-	// Offset to the configuration data.
-	unsigned int config = 0x3A;
-	if (parser->model == D4) {
-		config += 1;
-	} else if (parser->model == HELO2 || parser->model == D4i ||
-		parser->model == D6i || parser->model == D9tx ||
-		parser->model == DX) {
-		config = gasmix_offset + gasmix_count * 6;
-	}
-	if (config + 1 > size)
-		return DC_STATUS_DATAFORMAT;
+	// Cache the gas mix data.
+	dc_status_t rc = suunto_d9_parser_cache (parser);
+	if (rc != DC_STATUS_SUCCESS)
+		return rc;
 
 	// Number of parameters in the configuration data.
-	unsigned int nparams = data[config];
-	if (nparams == 0 || nparams > MAXPARAMS)
+	unsigned int nparams = data[parser->config];
+	if (nparams == 0 || nparams > MAXPARAMS) {
+		ERROR (abstract->context, "Invalid number of parameters.");
 		return DC_STATUS_DATAFORMAT;
+	}
 
 	// Available divisor values.
 	const unsigned int divisors[] = {1, 2, 4, 5, 10, 50, 100, 1000};
@@ -320,7 +423,7 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 	// Get the sample configuration.
 	sample_info_t info[MAXPARAMS] = {{0}};
 	for (unsigned int i = 0; i < nparams; ++i) {
-		unsigned int idx = config + 2 + i * 3;
+		unsigned int idx = parser->config + 2 + i * 3;
 		info[i].type     = data[idx + 0];
 		info[i].interval = data[idx + 1];
 		info[i].divisor  = divisors[(data[idx + 2] & 0x1C) >> 2];
@@ -333,32 +436,41 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 			info[i].size = 1;
 			break;
 		default: // Unknown sample type
+			ERROR (abstract->context, "Unknown sample type 0x%02x.", info[i].type);
 			return DC_STATUS_DATAFORMAT;
 		}
 	}
 
 	// Offset to the profile data.
-	unsigned int profile = config + 2 + nparams * 3;
-	if (profile + 5 > size)
+	unsigned int profile = parser->config + 2 + nparams * 3;
+	if (profile + 5 > size) {
+		ERROR (abstract->context, "Buffer overflow detected!");
 		return DC_STATUS_DATAFORMAT;
+	}
 
 	// HelO2 dives can have an additional data block.
 	const unsigned char sequence[] = {0x01, 0x00, 0x00};
 	if (parser->model == HELO2 && memcmp (data + profile, sequence, sizeof (sequence)) != 0)
 		profile += 12;
-	if (profile + 5 > size)
+	if (profile + 5 > size) {
+		ERROR (abstract->context, "Buffer overflow detected!");
 		return DC_STATUS_DATAFORMAT;
+	}
 
 	// Sample recording interval.
 	unsigned int interval_sample_offset = 0x18;
 	if (parser->model == HELO2 || parser->model == D4i ||
-		parser->model == D6i || parser->model == D9tx)
+		parser->model == D6i || parser->model == D9tx ||
+		parser->model == ZOOPNOVO || parser->model == VYPERNOVO ||
+		parser->model == D4F)
 		interval_sample_offset = 0x1E;
 	else if (parser->model == DX)
 		interval_sample_offset = 0x22;
 	unsigned int interval_sample = data[interval_sample_offset];
-	if (interval_sample == 0)
+	if (interval_sample == 0) {
+		ERROR (abstract->context, "Invalid sample interval.");
 		return DC_STATUS_DATAFORMAT;
+	}
 
 	// Offset to the first marker position.
 	unsigned int marker = array_uint16_le (data + profile + 3);
@@ -377,8 +489,10 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 		// Sample data.
 		for (unsigned int i = 0; i < nparams; ++i) {
 			if (info[i].interval && (nsamples % info[i].interval) == 0) {
-				if (offset + info[i].size > size)
+				if (offset + info[i].size > size) {
+					ERROR (abstract->context, "Buffer overflow detected!");
 					return DC_STATUS_DATAFORMAT;
+				}
 
 				unsigned int value = 0;
 				switch (info[i].type) {
@@ -400,6 +514,7 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 					if (callback) callback (DC_SAMPLE_TEMPERATURE, sample, userdata);
 					break;
 				default: // Unknown sample type
+					ERROR (abstract->context, "Unknown sample type 0x%02x.", info[i].type);
 					return DC_STATUS_DATAFORMAT;
 				}
 
@@ -407,31 +522,49 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 			}
 		}
 
+		// Initial gasmix.
+		if (time == 0 && parser->ngasmixes > 0) {
+			if (parser->gasmix >= parser->ngasmixes) {
+				ERROR (abstract->context, "Invalid initial gas mix.");
+				return DC_STATUS_DATAFORMAT;
+			}
+			sample.gasmix = parser->gasmix;
+			if (callback) callback (DC_SAMPLE_GASMIX, sample, userdata);
+		}
+
 		// Events
 		if ((nsamples + 1) == marker) {
 			while (offset < size) {
 				unsigned int event = data[offset++];
-				unsigned int seconds, type, unknown, heading, percentage;
+				unsigned int seconds, type, unknown, heading;
 				unsigned int current, next;
-				unsigned int he, o2;
+				unsigned int he, o2, idx;
+				unsigned int length;
 
+				sample.event.type = SAMPLE_EVENT_NONE;
 				sample.event.time = 0;
 				sample.event.flags = 0;
 				sample.event.value = 0;
 				switch (event) {
 				case 0x01: // Next Event Marker
-					if (offset + 4 > size)
+					if (offset + 4 > size) {
+						ERROR (abstract->context, "Buffer overflow detected!");
 						return DC_STATUS_DATAFORMAT;
+					}
 					current = array_uint16_le (data + offset + 0);
 					next    = array_uint16_le (data + offset + 2);
-					if (marker != current)
+					if (marker != current) {
+						ERROR (abstract->context, "Unexpected event marker!");
 						return DC_STATUS_DATAFORMAT;
+					}
 					marker += next;
 					offset += 4;
 					break;
 				case 0x02: // Surfaced
-					if (offset + 2 > size)
+					if (offset + 2 > size) {
+						ERROR (abstract->context, "Buffer overflow detected!");
 						return DC_STATUS_DATAFORMAT;
+					}
 					unknown = data[offset + 0];
 					seconds = data[offset + 1];
 					sample.event.type = SAMPLE_EVENT_SURFACE;
@@ -440,34 +573,36 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 					offset += 2;
 					break;
 				case 0x03: // Event
-					if (offset + 2 > size)
+					if (offset + 2 > size) {
+						ERROR (abstract->context, "Buffer overflow detected!");
 						return DC_STATUS_DATAFORMAT;
+					}
 					type    = data[offset + 0];
 					seconds = data[offset + 1];
 					switch (type & 0x7F) {
 					case 0x00: // Voluntary Safety Stop
-						sample.event.type = SAMPLE_EVENT_SAFETYSTOP_VOLUNTARY;
+						sample.event.type = SAMPLE_EVENT_NONE;
 						if (type & 0x80)
 							in_deco &= ~SAFETYSTOP;
 						else
 							in_deco |= SAFETYSTOP;
 						break;
 					case 0x01: // Mandatory Safety Stop - odd concept; model as deco stop
-						sample.event.type = SAMPLE_EVENT_SAFETYSTOP_MANDATORY;
+						sample.event.type = SAMPLE_EVENT_NONE;
 						if (type & 0x80)
 							in_deco &= ~DECOSTOP;
 						else
 							in_deco |= DECOSTOP;
 						break;
 					case 0x02: // Deep Safety Stop
-						sample.event.type = SAMPLE_EVENT_DEEPSTOP;
+						sample.event.type = SAMPLE_EVENT_NONE;
 						if (type & 0x80)
 							in_deco &= ~DEEPSTOP;
 						else
 							in_deco |= DEEPSTOP;
 						break;
 					case 0x03: // Deco
-						sample.event.type = SAMPLE_EVENT_DECOSTOP;
+						sample.event.type = SAMPLE_EVENT_NONE;
 						if (type & 0x80)
 							in_deco &= ~DECOSTOP;
 						else
@@ -517,21 +652,21 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 						sample.event.type = SAMPLE_EVENT_TISSUELEVEL;
 						break;
 					case 0x13: // Deep Safety Stop
-						sample.event.type = SAMPLE_EVENT_DEEPSTOP;
+						sample.event.type = SAMPLE_EVENT_NONE;
 						if (type & 0x80)
 							in_deco &= ~DEEPSTOP;
 						else
 							in_deco |= DEEPSTOP;
 						break;
 					case 0x14: // Mandatory Safety Stop - again, model as deco stop
-						sample.event.type = SAMPLE_EVENT_SAFETYSTOP_MANDATORY;
+						sample.event.type = SAMPLE_EVENT_NONE;
 						if (type & 0x80)
 							in_deco &= ~DECOSTOP;
 						else
 							in_deco |= DECOSTOP;
 						break;
 					default: // Unknown
-						WARNING (abstract->context, "Unknown event");
+						WARNING (abstract->context, "Unknown event type 0x%02x.", type);
 						break;
 					}
 					if (type & 0x80)
@@ -539,12 +674,16 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 					else
 						sample.event.flags = SAMPLE_FLAGS_BEGIN;
 					sample.event.time = seconds;
-					if (callback) callback (DC_SAMPLE_EVENT, sample, userdata);
+					if (sample.event.type != SAMPLE_EVENT_NONE) {
+						if (callback) callback (DC_SAMPLE_EVENT, sample, userdata);
+					}
 					offset += 2;
 					break;
 				case 0x04: // Bookmark/Heading
-					if (offset + 4 > size)
+					if (offset + 4 > size) {
+						ERROR (abstract->context, "Buffer overflow detected!");
 						return DC_STATUS_DATAFORMAT;
+					}
 					unknown = data[offset + 0];
 					seconds = data[offset + 1];
 					heading = array_uint16_le (data + offset + 2);
@@ -560,31 +699,49 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 					offset += 4;
 					break;
 				case 0x05: // Gas Change
-					if (offset + 2 > size)
+					if (offset + 2 > size) {
+						ERROR (abstract->context, "Buffer overflow detected!");
 						return DC_STATUS_DATAFORMAT;
-					percentage = data[offset + 0];
+					}
+					o2 = data[offset + 0];
 					seconds = data[offset + 1];
-					sample.event.type = SAMPLE_EVENT_GASCHANGE;
-					sample.event.time = seconds;
-					sample.event.value = percentage;
-					if (callback) callback (DC_SAMPLE_EVENT, sample, userdata);
+					idx = suunto_d9_parser_find_gasmix(parser, o2, 0);
+					if (idx >= parser->ngasmixes) {
+						ERROR (abstract->context, "Invalid gas mix.");
+						return DC_STATUS_DATAFORMAT;
+					}
+					sample.gasmix = idx;
+					if (callback) callback (DC_SAMPLE_GASMIX, sample, userdata);
 					offset += 2;
 					break;
 				case 0x06: // Gas Change
-					if (offset + 4 > size)
+					if (parser->model == DX || parser->model == VYPERNOVO)
+						length = 5;
+					else
+						length = 4;
+					if (offset + length > size) {
+						ERROR (abstract->context, "Buffer overflow detected!");
 						return DC_STATUS_DATAFORMAT;
+					}
 					unknown = data[offset + 0];
 					he = data[offset + 1];
 					o2 = data[offset + 2];
-					seconds = data[offset + 3];
-					sample.event.type = SAMPLE_EVENT_GASCHANGE2;
-					sample.event.time = seconds;
-					sample.event.value = o2 | (he << 16);
-					if (callback) callback (DC_SAMPLE_EVENT, sample, userdata);
-					offset += 4;
+					if (parser->model == DX || parser->model == VYPERNOVO) {
+						seconds = data[offset + 4];
+					} else {
+						seconds = data[offset + 3];
+					}
+					idx = suunto_d9_parser_find_gasmix(parser, o2, he);
+					if (idx >= parser->ngasmixes) {
+						ERROR (abstract->context, "Invalid gas mix.");
+						return DC_STATUS_DATAFORMAT;
+					}
+					sample.gasmix = idx;
+					if (callback) callback (DC_SAMPLE_GASMIX, sample, userdata);
+					offset += length;
 					break;
 				default:
-					WARNING (abstract->context, "Unknown event");
+					WARNING (abstract->context, "Unknown event 0x%02x.", event);
 					break;
 				}
 
