@@ -1,18 +1,18 @@
-/* 
+/*
  * libdivecomputer
- * 
+ *
  * Copyright (C) 2008 Jef Driesen
- * 
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
@@ -23,22 +23,16 @@
 #include <stdlib.h> // malloc, free
 #include <assert.h>	// assert
 
-#include <libdivecomputer/suunto_vyper.h>
-
+#include "suunto_vyper.h"
+#include "suunto_common.h"
 #include "context-private.h"
 #include "device-private.h"
-#include "suunto_common.h"
 #include "serial.h"
 #include "checksum.h"
 #include "array.h"
 
 #define ISINSTANCE(device) dc_device_isinstance((device), &suunto_vyper_device_vtable)
 
-#define EXITCODE(rc) \
-( \
-	rc == -1 ? DC_STATUS_IO : DC_STATUS_TIMEOUT \
-)
-
 #define MIN(a,b)	(((a) < (b)) ? (a) : (b))
 #define MAX(a,b)	(((a) > (b)) ? (a) : (b))
 
@@ -52,7 +46,7 @@
 
 typedef struct suunto_vyper_device_t {
 	suunto_common_device_t base;
-	serial_t *port;
+	dc_serial_t *port;
 } suunto_vyper_device_t;
 
 static dc_status_t suunto_vyper_device_read (dc_device_t *abstract, unsigned int address, unsigned char data[], unsigned int size);
@@ -62,12 +56,14 @@ static dc_status_t suunto_vyper_device_foreach (dc_device_t *abstract, dc_dive_c
 static dc_status_t suunto_vyper_device_close (dc_device_t *abstract);
 
 static const dc_device_vtable_t suunto_vyper_device_vtable = {
+	sizeof(suunto_vyper_device_t),
 	DC_FAMILY_SUUNTO_VYPER,
 	suunto_common_device_set_fingerprint, /* set_fingerprint */
 	suunto_vyper_device_read, /* read */
 	suunto_vyper_device_write, /* write */
 	suunto_vyper_device_dump, /* dump */
 	suunto_vyper_device_foreach, /* foreach */
+	NULL, /* timesync */
 	suunto_vyper_device_close /* close */
 };
 
@@ -91,118 +87,122 @@ static const suunto_common_layout_t suunto_spyder_layout = {
 dc_status_t
 suunto_vyper_device_open (dc_device_t **out, dc_context_t *context, const char *name)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
+	suunto_vyper_device_t *device = NULL;
+
 	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
-	suunto_vyper_device_t *device = (suunto_vyper_device_t *) malloc (sizeof (suunto_vyper_device_t));
+	device = (suunto_vyper_device_t *) dc_device_allocate (context, &suunto_vyper_device_vtable);
 	if (device == NULL) {
 		ERROR (context, "Failed to allocate memory.");
 		return DC_STATUS_NOMEMORY;
 	}
 
 	// Initialize the base class.
-	suunto_common_device_init (&device->base, context, &suunto_vyper_device_vtable);
+	suunto_common_device_init (&device->base);
 
 	// Set the default values.
 	device->port = NULL;
 
 	// Open the device.
-	int rc = serial_open (&device->port, context, name);
-	if (rc == -1) {
+	status = dc_serial_open (&device->port, context, name);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to open the serial port.");
-		free (device);
-		return DC_STATUS_IO;
+		goto error_free;
 	}
 
 	// Set the serial communication protocol (2400 8O1).
-	rc = serial_configure (device->port, 2400, 8, SERIAL_PARITY_ODD, 1, SERIAL_FLOWCONTROL_NONE);
-	if (rc == -1) {
+	status = dc_serial_configure (device->port, 2400, 8, DC_PARITY_ODD, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the terminal attributes.");
-		serial_close (device->port);
-		free (device);
-		return DC_STATUS_IO;
+		goto error_close;
 	}
 
 	// Set the timeout for receiving data (1000 ms).
-	if (serial_set_timeout (device->port, 1000) == -1) {
+	status = dc_serial_set_timeout (device->port, 1000);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the timeout.");
-		serial_close (device->port);
-		free (device);
-		return DC_STATUS_IO;
+		goto error_close;
 	}
 
 	// Set the DTR line (power supply for the interface).
-	if (serial_set_dtr (device->port, 1) == -1) {
+	status = dc_serial_set_dtr (device->port, 1);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to set the DTR line.");
-		serial_close (device->port);
-		free (device);
-		return DC_STATUS_IO;
+		goto error_close;
 	}
 
 	// Give the interface 100 ms to settle and draw power up.
-	serial_sleep (device->port, 100);
+	dc_serial_sleep (device->port, 100);
 
 	// Make sure everything is in a sane state.
-	serial_flush (device->port, SERIAL_QUEUE_BOTH);
+	dc_serial_purge (device->port, DC_DIRECTION_ALL);
 
 	*out = (dc_device_t*) device;
 
 	return DC_STATUS_SUCCESS;
+
+error_close:
+	dc_serial_close (device->port);
+error_free:
+	dc_device_deallocate ((dc_device_t *) device);
+	return status;
 }
 
 
 static dc_status_t
 suunto_vyper_device_close (dc_device_t *abstract)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
 	suunto_vyper_device_t *device = (suunto_vyper_device_t*) abstract;
+	dc_status_t rc = DC_STATUS_SUCCESS;
 
 	// Close the device.
-	if (serial_close (device->port) == -1) {
-		free (device);
-		return DC_STATUS_IO;
+	rc = dc_serial_close (device->port);
+	if (rc != DC_STATUS_SUCCESS) {
+		dc_status_set_error(&status, rc);
 	}
 
-	// Free memory.	
-	free (device);
-
-	return DC_STATUS_SUCCESS;
+	return status;
 }
 
 
 static dc_status_t
 suunto_vyper_send (suunto_vyper_device_t *device, const unsigned char command[], unsigned int csize)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
 	dc_device_t *abstract = (dc_device_t *) device;
 
-	serial_sleep (device->port, 500);
+	dc_serial_sleep (device->port, 500);
 
 	// Set RTS to send the command.
-	serial_set_rts (device->port, 1);
+	dc_serial_set_rts (device->port, 1);
 
 	// Send the command to the dive computer.
-	int n = serial_write (device->port, command, csize);
-	if (n != csize) {
+	status = dc_serial_write (device->port, command, csize, NULL);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
-		return EXITCODE (n);
+		return status;
 	}
 
-	// If the interface sends an echo back (which is the case for many clone 
-	// interfaces), this echo should be removed from the input queue before 
-	// attempting to read the real reply from the dive computer. Otherwise, 
-	// the data transfer will fail. Timing is also critical here! We have to 
-	// wait at least until the echo appears (40ms), but not until the reply 
+	// If the interface sends an echo back (which is the case for many clone
+	// interfaces), this echo should be removed from the input queue before
+	// attempting to read the real reply from the dive computer. Otherwise,
+	// the data transfer will fail. Timing is also critical here! We have to
+	// wait at least until the echo appears (40ms), but not until the reply
 	// from the dive computer appears (600ms).
-	// The original suunto interface does not have this problem, because it 
-	// does not send an echo and the RTS switching makes it impossible to 
-	// receive the reply before RTS is cleared. We have to wait some time 
-	// before clearing RTS (around 30ms). But if we wait too long (> 500ms), 
+	// The original suunto interface does not have this problem, because it
+	// does not send an echo and the RTS switching makes it impossible to
+	// receive the reply before RTS is cleared. We have to wait some time
+	// before clearing RTS (around 30ms). But if we wait too long (> 500ms),
 	// the reply disappears again.
-	serial_sleep (device->port, 200);
-	serial_flush (device->port, SERIAL_QUEUE_INPUT);
+	dc_serial_sleep (device->port, 200);
+	dc_serial_purge (device->port, DC_DIRECTION_INPUT);
 
 	// Clear RTS to receive the reply.
-	serial_set_rts (device->port, 0);
+	dc_serial_set_rts (device->port, 0);
 
 	return DC_STATUS_SUCCESS;
 }
@@ -211,10 +211,11 @@ suunto_vyper_send (suunto_vyper_device_t *device, const unsigned char command[],
 static dc_status_t
 suunto_vyper_transfer (suunto_vyper_device_t *device, const unsigned char command[], unsigned int csize, unsigned char answer[], unsigned int asize, unsigned int size)
 {
-	assert (asize >= size + 2);
-
+	dc_status_t status = DC_STATUS_SUCCESS;
 	dc_device_t *abstract = (dc_device_t *) device;
 
+	assert (asize >= size + 2);
+
 	if (device_is_cancelled (abstract))
 		return DC_STATUS_CANCELLED;
 
@@ -226,10 +227,10 @@ suunto_vyper_transfer (suunto_vyper_device_t *device, const unsigned char comman
 	}
 
 	// Receive the answer of the dive computer.
-	int n = serial_read (device->port, answer, asize);
-	if (n != asize) {
+	status = dc_serial_read (device->port, answer, asize, NULL);
+	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
-		return EXITCODE (n);
+		return status;
 	}
 
 	// Verify the header of the package.
@@ -325,6 +326,7 @@ suunto_vyper_device_write (dc_device_t *abstract, unsigned int address, const un
 static dc_status_t
 suunto_vyper_read_dive (dc_device_t *abstract, dc_buffer_t *buffer, int init, dc_event_progress_t *progress)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
 	suunto_vyper_device_t *device = (suunto_vyper_device_t*) abstract;
 
 	if (device_is_cancelled (abstract))
@@ -348,25 +350,26 @@ suunto_vyper_read_dive (dc_device_t *abstract, dc_buffer_t *buffer, int init, dc
 	unsigned int nbytes = 0;
 	for (unsigned int npackages = 0;; ++npackages) {
 		// Receive the header of the package.
+		size_t n = 0;
 		unsigned char answer[SZ_PACKET + 3] = {0};
-		int n = serial_read (device->port, answer, 2);
-		if (n != 2) {
-			// If no data is received because a timeout occured, we assume 
-			// the last package was already received and the transmission 
-			// can be finished. Unfortunately this is not 100% reliable, 
-			// because there is always a small chance that more data will 
-			// arrive later (especially with a short timeout). But it works 
+		status = dc_serial_read (device->port, answer, 2, &n);
+		if (status != DC_STATUS_SUCCESS) {
+			// If no data is received because a timeout occured, we assume
+			// the last package was already received and the transmission
+			// can be finished. Unfortunately this is not 100% reliable,
+			// because there is always a small chance that more data will
+			// arrive later (especially with a short timeout). But it works
 			// good enough in practice.
-			// Only for the very first package, we can be sure there was 
+			// Only for the very first package, we can be sure there was
 			// an error, because the DC always sends at least one package.
 			if (n == 0 && npackages != 0)
 				break;
 			ERROR (abstract->context, "Failed to receive the answer.");
-			return EXITCODE (n);
+			return status;
 		}
 
 		// Verify the header of the package.
-		if (answer[0] != command[0] || 
+		if (answer[0] != command[0] ||
 			answer[1] > SZ_PACKET) {
 			ERROR (abstract->context, "Unexpected answer start byte(s).");
 			return DC_STATUS_PROTOCOL;
@@ -374,10 +377,10 @@ suunto_vyper_read_dive (dc_device_t *abstract, dc_buffer_t *buffer, int init, dc
 
 		// Receive the remaining part of the package.
 		unsigned char len = answer[1];
-		n = serial_read (device->port, answer + 2, len + 1);
-		if (n != len + 1) {
+		status = dc_serial_read (device->port, answer + 2, len + 1, NULL);
+		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to receive the answer.");
-			return EXITCODE (n);
+			return status;
 		}
 
 		// Verify the checksum of the package.
@@ -388,9 +391,9 @@ suunto_vyper_read_dive (dc_device_t *abstract, dc_buffer_t *buffer, int init, dc
 			return DC_STATUS_PROTOCOL;
 		}
 
-		// The DC sends a null package (a package with length zero) when it 
-		// has reached the end of its internal ring buffer. From this point on, 
-		// the current dive has been overwritten with newer data. Therefore, 
+		// The DC sends a null package (a package with length zero) when it
+		// has reached the end of its internal ring buffer. From this point on,
+		// the current dive has been overwritten with newer data. Therefore,
 		// we discard the current (incomplete) dive and end the transmission.
 		if (len == 0) {
 			dc_buffer_clear (buffer);
@@ -415,8 +418,8 @@ suunto_vyper_read_dive (dc_device_t *abstract, dc_buffer_t *buffer, int init, dc
 		nbytes += len;
 
 		// If a package is smaller than $SZ_PACKET bytes,
-		// we assume it's the last packet and the transmission can be 
-		// finished early. However, this approach does not work if the 
+		// we assume it's the last packet and the transmission can be
+		// finished early. However, this approach does not work if the
 		// last packet is exactly $SZ_PACKET bytes long!
 #if 0
 		if (len != SZ_PACKET)
@@ -430,9 +433,9 @@ suunto_vyper_read_dive (dc_device_t *abstract, dc_buffer_t *buffer, int init, dc
 		return DC_STATUS_NOMEMORY;
 	}
 
-	// The DC traverses its internal ring buffer backwards. The most recent 
-	// dive is send first (which allows you to download only the new dives), 
-	// but also the contents of each dive is reversed. Therefore, we reverse 
+	// The DC traverses its internal ring buffer backwards. The most recent
+	// dive is send first (which allows you to download only the new dives),
+	// but also the contents of each dive is reversed. Therefore, we reverse
 	// the bytes again before returning them to the application.
 	array_reverse_bytes (dc_buffer_get_data (buffer), dc_buffer_get_size (buffer));
 
@@ -440,16 +443,6 @@ suunto_vyper_read_dive (dc_device_t *abstract, dc_buffer_t *buffer, int init, dc
 }
 
 
-dc_status_t
-suunto_vyper_device_read_dive (dc_device_t *abstract, dc_buffer_t *buffer, int init)
-{
-	if (!ISINSTANCE (abstract))
-		return DC_STATUS_INVALIDARGS;
-
-	return suunto_vyper_read_dive (abstract, buffer, init, NULL);
-}
-
-
 static dc_status_t
 suunto_vyper_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 {
@@ -551,22 +544,3 @@ suunto_vyper_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback,
 
 	return rc;
 }
-
-
-dc_status_t
-suunto_vyper_extract_dives (dc_device_t *abstract, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata)
-{
-	suunto_common_device_t *device = (suunto_common_device_t*) abstract;
-
-	if (abstract && !ISINSTANCE (abstract))
-		return DC_STATUS_INVALIDARGS;
-
-	if (size < SZ_MEMORY)
-		return DC_STATUS_DATAFORMAT;
-
-	const suunto_common_layout_t *layout = &suunto_vyper_layout;
-	if (data[HDR_DEVINFO_VYPER] == 20 || data[HDR_DEVINFO_VYPER] == 30 || data[HDR_DEVINFO_VYPER] == 60)
-		layout = &suunto_spyder_layout;
-
-	return suunto_common_extract_dives (device, layout, data, callback, userdata);
-}
