@@ -22,17 +22,15 @@
 #ifndef ZEAGLE_N2ITION3_H
 #define ZEAGLE_N2ITION3_H
 
-#include "device.h"
+#include <libdivecomputer/context.h>
+#include <libdivecomputer/device.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
-#define ZEAGLE_N2ITION3_MEMORY_SIZE 0x8000
-#define ZEAGLE_N2ITION3_PACKET_SIZE 64
-
-device_status_t
-zeagle_n2ition3_device_open (device_t **device, const char* name);
+dc_status_t
+zeagle_n2ition3_device_open (dc_device_t **device, dc_context_t *context, const char *name);
 
 #ifdef __cplusplus
 }
